@@ -309,6 +309,7 @@ public class HTTPClientDocs
             .onRequestContent((request, content) -> { /* ... */ })
             .onRequestFailure((request, failure) -> { /* ... */ })
             .onRequestSuccess(request -> { /* ... */ })
+            .onRequestListener(new Request.Listener() {}) // For all request events.
             // Add response hooks.
             .onResponseBegin(response -> { /* ... */ })
             .onResponseHeader((response, field) -> true)
@@ -316,6 +317,7 @@ public class HTTPClientDocs
             .onResponseContentAsync((response, chunk, demander) -> demander.run())
             .onResponseFailure((response, failure) -> { /* ... */ })
             .onResponseSuccess(response -> { /* ... */ })
+            .onResponseListener(new Response.Listener() {}) // For all response events.
             // Result hook.
             .send(result -> { /* ... */ });
         // end::listeners[]
@@ -475,25 +477,29 @@ public class HTTPClientDocs
         httpClient.start();
 
         // tag::inputStreamResponseListener[]
-        InputStreamResponseListener listener = new InputStreamResponseListener();
-        httpClient.newRequest("http://domain.com/path")
-            .send(listener);
-
-        // Wait for the response headers to arrive.
-        Response response = listener.get(5, TimeUnit.SECONDS);
-
-        // Look at the response before streaming the content.
-        if (response.getStatus() == HttpStatus.OK_200)
+        // Use try-with-resources to make sure that resources
+        // are released even if the InputStream is not used.
+        try (InputStreamResponseListener listener = new InputStreamResponseListener())
         {
-            // Use try-with-resources to close input stream.
-            try (InputStream responseContent = listener.getInputStream())
+            httpClient.newRequest("http://domain.com/path")
+                .send(listener);
+
+            // Wait for the response headers to arrive.
+            Response response = listener.get(5, TimeUnit.SECONDS);
+
+            // Look at the response before streaming the content.
+            if (response.getStatus() == HttpStatus.OK_200)
             {
-                // Your logic here
+                // Use try-with-resources to close input stream.
+                try (InputStream responseContent = listener.getInputStream())
+                {
+                    // Your logic here
+                }
+            }
+            else
+            {
+                response.abort(new IOException("Unexpected HTTP response"));
             }
-        }
-        else
-        {
-            response.abort(new IOException("Unexpected HTTP response"));
         }
         // end::inputStreamResponseListener[]
     }
