@@ -29,6 +29,8 @@ import jakarta.servlet.ServletInputStream;
 import jakarta.servlet.http.HttpServlet;
 import jakarta.servlet.http.HttpServletRequest;
 import jakarta.servlet.http.HttpServletResponse;
+import org.bouncycastle.jce.provider.BouncyCastleProvider;
+import org.bouncycastle.jsse.provider.BouncyCastleJsseProvider;
 import org.conscrypt.OpenSSLProvider;
 import org.eclipse.jetty.alpn.server.ALPNServerConnectionFactory;
 import org.eclipse.jetty.client.ContentResponse;
@@ -60,7 +62,6 @@ import org.eclipse.jetty.rewrite.handler.RedirectRegexRule;
 import org.eclipse.jetty.rewrite.handler.RewriteHandler;
 import org.eclipse.jetty.rewrite.handler.RewriteRegexRule;
 import org.eclipse.jetty.server.ConnectionFactory;
-import org.eclipse.jetty.server.ConnectionLimit;
 import org.eclipse.jetty.server.Connector;
 import org.eclipse.jetty.server.CustomRequestLog;
 import org.eclipse.jetty.server.FormFields;
@@ -69,6 +70,7 @@ import org.eclipse.jetty.server.HttpConfiguration;
 import org.eclipse.jetty.server.HttpConnectionFactory;
 import org.eclipse.jetty.server.MemoryConnector;
 import org.eclipse.jetty.server.MemoryTransport;
+import org.eclipse.jetty.server.NetworkConnectionLimit;
 import org.eclipse.jetty.server.NetworkConnector;
 import org.eclipse.jetty.server.ProxyConnectionFactory;
 import org.eclipse.jetty.server.Request;
@@ -98,6 +100,7 @@ import org.eclipse.jetty.util.NanoTime;
 import org.eclipse.jetty.util.Promise;
 import org.eclipse.jetty.util.resource.Resource;
 import org.eclipse.jetty.util.resource.ResourceFactory;
+import org.eclipse.jetty.util.ssl.KeyStoreScanner;
 import org.eclipse.jetty.util.ssl.SslContextFactory;
 import org.eclipse.jetty.util.thread.QueuedThreadPool;
 
@@ -354,14 +357,14 @@ public class HTTPServerDocs
         // end::sameRandomPort[]
     }
 
-    public void connectionLimit()
+    public void networkConnectionLimit()
     {
-        // tag::connectionLimit[]
+        // tag::networkConnectionLimit[]
         Server server = new Server();
 
-        // Limit connections to the server, across all connectors.
-        ConnectionLimit serverConnectionLimit = new ConnectionLimit(1024, server);
-        server.addBean(serverConnectionLimit);
+        // Limit TCP connections to the server, across all connectors.
+        NetworkConnectionLimit serverNetworkConnectionLimit = new NetworkConnectionLimit(1024, server);
+        server.addBean(serverNetworkConnectionLimit);
 
         ServerConnector connector1 = new ServerConnector(server);
         connector1.setPort(8080);
@@ -371,9 +374,9 @@ public class HTTPServerDocs
         connector2.setPort(9090);
         server.addConnector(connector2);
         // Limit connections for this connector only.
-        ConnectionLimit connectorConnectionLimit = new ConnectionLimit(64, connector2);
-        connector2.addBean(connectorConnectionLimit);
-        // end::connectionLimit[]
+        NetworkConnectionLimit connectorNetworkConnectionLimit = new NetworkConnectionLimit(64, connector2);
+        connector2.addBean(connectorNetworkConnectionLimit);
+        // end::networkConnectionLimit[]
     }
 
     public void sslHandshakeListener() throws Exception
@@ -604,11 +607,61 @@ public class HTTPServerDocs
         SslContextFactory.Server sslContextFactory = new SslContextFactory.Server();
         sslContextFactory.setKeyStorePath("/path/to/keystore");
         sslContextFactory.setKeyStorePassword("secret");
-        // Configure Jetty's SslContextFactory to use Conscrypt.
+        // Configure Jetty's SslContextFactory to use the Conscrypt provider.
         sslContextFactory.setProvider("Conscrypt");
         // end::conscrypt[]
     }
 
+    public void bouncyCastle()
+    {
+        // tag::bouncyCastle[]
+        // Configure the JDK with the Bouncy Castle providers, you need both.
+        Security.addProvider(new BouncyCastleProvider());
+        Security.addProvider(new BouncyCastleJsseProvider());
+
+        SslContextFactory.Server sslContextFactory = new SslContextFactory.Server();
+        sslContextFactory.setKeyStorePath("/path/to/keystore");
+        sslContextFactory.setKeyStorePassword("secret");
+        // Configure Jetty's SslContextFactory to use the Bouncy Castle provider.
+        sslContextFactory.setProvider("BCJSSE");
+        // end::bouncyCastle[]
+    }
+
+    public void keyStoreScanner() throws Exception
+    {
+        // tag::keyStoreScanner[]
+        Server server = new Server();
+
+        // The HTTP configuration object.
+        HttpConfiguration httpConfig = new HttpConfiguration();
+        // Add the SecureRequestCustomizer because TLS is used.
+        httpConfig.addCustomizer(new SecureRequestCustomizer());
+
+        // The ConnectionFactory for HTTP/1.1.
+        HttpConnectionFactory http11 = new HttpConnectionFactory(httpConfig);
+
+        // Configure the SslContextFactory with the keyStore information.
+        SslContextFactory.Server sslContextFactory = new SslContextFactory.Server();
+        sslContextFactory.setKeyStorePath("/path/to/keystore");
+        sslContextFactory.setKeyStorePassword("secret");
+
+        // The ConnectionFactory for TLS.
+        SslConnectionFactory tls = new SslConnectionFactory(sslContextFactory, http11.getProtocol());
+
+        // The ServerConnector instance.
+        ServerConnector connector = new ServerConnector(server, tls, http11);
+        connector.setPort(8443);
+
+        server.addConnector(connector);
+
+        KeyStoreScanner keyStoreScanner = new KeyStoreScanner(sslContextFactory);
+        keyStoreScanner.setScanInterval(60);
+        server.addBean(keyStoreScanner);
+
+        server.start();
+        // end::keyStoreScanner[]
+    }
+
     public void handlerTree()
     {
         class LoggingHandler extends Handler.Abstract
