@@ -87,11 +87,14 @@ import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.awaitility.Awaitility.await;
 import static org.hamcrest.MatcherAssert.assertThat;
 import static org.hamcrest.Matchers.containsString;
+import static org.hamcrest.Matchers.greaterThan;
 import static org.hamcrest.Matchers.instanceOf;
+import static org.hamcrest.Matchers.is;
 import static org.hamcrest.Matchers.notNullValue;
 import static org.junit.jupiter.api.Assertions.assertArrayEquals;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertInstanceOf;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
@@ -801,8 +804,10 @@ public class HttpClientTest extends AbstractHttpClientServerTest
 
         assertThrows(IOException.class, () ->
         {
-            Socket socket = new Socket();
-            socket.connect(new InetSocketAddress(host, port), 1000);
+            try (Socket socket = new Socket())
+            {
+                socket.connect(new InetSocketAddress(host, port), 1000);
+            }
         }, "Host must not be resolvable");
 
         start(scenario, new EmptyServerHandler());
@@ -813,7 +818,7 @@ public class HttpClientTest extends AbstractHttpClientServerTest
             {
                 assertTrue(result.isFailed());
                 Throwable failure = result.getFailure();
-                assertTrue(failure instanceof UnknownHostException);
+                assertInstanceOf(UnknownHostException.class, failure);
                 latch.countDown();
             });
         assertTrue(latch.await(10, TimeUnit.SECONDS));
@@ -999,7 +1004,7 @@ public class HttpClientTest extends AbstractHttpClientServerTest
             .onRequestContent(listener)
             .onRequestSuccess(listener)
             .onRequestFailure(listener)
-            .listener(listener)
+            .onRequestListener(listener)
             .send();
 
         assertEquals(200, response.getStatus());
@@ -1092,6 +1097,44 @@ public class HttpClientTest extends AbstractHttpClientServerTest
         assertEquals(expected, counter.get());
     }
 
+    @ParameterizedTest
+    @ArgumentsSource(ScenarioProvider.class)
+    public void testRequestListenerResponseListener(Scenario scenario) throws Exception
+    {
+        start(scenario, new EmptyServerHandler());
+
+        class MetricsListener implements Request.Listener, Response.Listener
+        {
+            private long responseNanoTime;
+
+            @Override
+            public void onQueued(Request request)
+            {
+                request.attribute("queued", NanoTime.now());
+            }
+
+            @Override
+            public void onComplete(Result result)
+            {
+                long queuedNanoTime = (Long)result.getRequest().getAttributes().get("queued");
+                responseNanoTime = NanoTime.since(queuedNanoTime);
+            }
+        }
+
+        MetricsListener metricsListener = new MetricsListener();
+
+        Request request = client.newRequest("localhost", connector.getLocalPort())
+            .scheme(scenario.getScheme())
+            // Separate orthogonal concerns such as metrics (e.g. responseTime)
+            // from the handling of the response (e.g. accumulate response content).
+            .onRequestListener(metricsListener)
+            .onResponseListener(metricsListener);
+        ContentResponse response = new CompletableResponseListener(request).send().get(5, TimeUnit.SECONDS);
+
+        assertThat(response.getStatus(), is(HttpStatus.OK_200));
+        assertThat(metricsListener.responseNanoTime, greaterThan(0L));
+    }
+
     @ParameterizedTest
     @ArgumentsSource(ScenarioProvider.class)
     public void setOnCompleteCallbackWithBlockingSend(Scenario scenario) throws Exception
@@ -1506,9 +1549,10 @@ public class HttpClientTest extends AbstractHttpClientServerTest
                 consume(input, false);
 
                 // HTTP/1.0 response, the client must not close the connection.
-                String httpResponse =
-                    "HTTP/1.0 200 OK\r\n" +
-                        "\r\n";
+                String httpResponse = """
+                    HTTP/1.0 200 OK
+                    
+                    """;
                 OutputStream output = socket.getOutputStream();
                 output.write(httpResponse.getBytes(UTF_8));
                 output.flush();
@@ -1532,10 +1576,11 @@ public class HttpClientTest extends AbstractHttpClientServerTest
 
                 consume(input, false);
 
-                httpResponse =
-                    "HTTP/1.1 200 OK\r\n" +
-                        "Content-Length: 0\r\n" +
-                        "\r\n";
+                httpResponse = """
+                    HTTP/1.1 200 OK
+                    Content-Length: 0
+                    
+                    """;
                 output.write(httpResponse.getBytes(UTF_8));
                 output.flush();
 
@@ -1676,10 +1721,10 @@ public class HttpClientTest extends AbstractHttpClientServerTest
                 consume(input, false);
 
                 // Send a bad response.
-                String httpResponse =
-                    "HTTP/1.1 204 No Content\r\n" +
-                        "\r\n" +
-                        "No Content";
+                String httpResponse = """
+                    HTTP/1.1 204 No Content
+                    
+                    No Content""";
                 OutputStream output = socket.getOutputStream();
                 output.write(httpResponse.getBytes(UTF_8));
                 output.flush();
@@ -1699,10 +1744,11 @@ public class HttpClientTest extends AbstractHttpClientServerTest
 
                 consume(input, false);
 
-                httpResponse =
-                    "HTTP/1.1 200 OK\r\n" +
-                        "Content-Length: 0\r\n" +
-                        "\r\n";
+                httpResponse = """
+                    HTTP/1.1 200 OK
+                    Content-Length: 0
+                    
+                    """;
                 output.write(httpResponse.getBytes(UTF_8));
                 output.flush();
 
@@ -1766,11 +1812,12 @@ public class HttpClientTest extends AbstractHttpClientServerTest
     @ArgumentsSource(ScenarioProvider.class)
     public void testUnsolicitedResponseBytesFromServer(Scenario scenario) throws Exception
     {
-        String response = "" +
-            "HTTP/1.1 408 Request Timeout\r\n" +
-            "Content-Length: 0\r\n" +
-            "Connection: close\r\n" +
-            "\r\n";
+        String response = """
+            HTTP/1.1 408 Request Timeout
+            Content-Length: 0
+            Connection: close
+            
+            """;
         testUnsolicitedBytesFromServer(scenario, response);
     }
 
@@ -1905,6 +1952,18 @@ public class HttpClientTest extends AbstractHttpClientServerTest
     public void testBindAddress(Scenario scenario) throws Exception
     {
         String bindAddress = "127.0.0.2";
+
+        // Pre-check if the address is bindable (on macOS probably not)
+        try (ServerSocket testSocket = new ServerSocket())
+        {
+            testSocket.bind(new InetSocketAddress(bindAddress, 0));
+        }
+        catch (IOException e)
+        {
+            Assumptions.assumeTrue(false, 
+                "Cannot bind to " + bindAddress + " address on this system");
+        }
+
         start(scenario, new EmptyServerHandler()
         {
             @Override
@@ -2012,6 +2071,33 @@ public class HttpClientTest extends AbstractHttpClientServerTest
             .send());
     }
 
+    @ParameterizedTest
+    @ArgumentsSource(ScenarioProvider.class)
+    public void testMaxRequestHeadersSizeSmallerThanRequestHeadersSize(Scenario scenario) throws Exception
+    {
+        start(scenario, new EmptyServerHandler());
+
+        RetainableByteBuffer buffer = client.getByteBufferPool().acquire(client.getRequestBufferSize(), false);
+        int capacity = buffer.capacity();
+        buffer.release();
+        client.setMaxRequestHeadersSize(capacity / 4);
+
+        ContentResponse response = client.newRequest("localhost", connector.getLocalPort())
+            .scheme(scenario.getScheme())
+            .agent("A".repeat((capacity / 8)))
+            .timeout(5, TimeUnit.SECONDS)
+            .send();
+
+        assertEquals(HttpStatus.OK_200, response.getStatus());
+
+        assertThrows(ExecutionException.class, () -> client.newRequest("localhost", connector.getLocalPort())
+            .scheme(scenario.getScheme())
+            // Overflow the max request headers size.
+            .agent("A".repeat(capacity / 2))
+            .timeout(5, TimeUnit.SECONDS)
+            .send());
+    }
+
     @ParameterizedTest
     @ArgumentsSource(ScenarioProvider.class)
     public void testMaxResponseHeadersSize(Scenario scenario) throws Exception
@@ -2019,7 +2105,7 @@ public class HttpClientTest extends AbstractHttpClientServerTest
         start(scenario, new EmptyServerHandler()
         {
             @Override
-            protected void service(org.eclipse.jetty.server.Request request, org.eclipse.jetty.server.Response response) throws Throwable
+            protected void service(org.eclipse.jetty.server.Request request, org.eclipse.jetty.server.Response response)
             {
                 int capacity = (int)request.getHeaders().getLongField("X-Capacity");
                 // Overflow the max request headers size, should generate a 500.
