@@ -429,8 +429,8 @@ public class HttpFieldsTest
         BufferUtil.flipToFlush(buffer, 0);
         String out = BufferUtil.toString(buffer);
         assertThat(out, containsString("name0: value  0"));
-        assertThat(out, containsString("name??1: value1"));
-        assertThat(out, containsString("name?2: value:  2"));
+        assertThat(out, containsString("name..1: value1"));
+        assertThat(out, containsString("name.2: value:  2"));
     }
 
     @ParameterizedTest
@@ -1020,8 +1020,6 @@ public class HttpFieldsTest
     {
         HttpFields.Mutable fields = HttpFields.build();
         assertThrows(NullPointerException.class, () -> fields.add((String)null, "bogus"));
-        assertThat(fields.size(), is(0));
-
         assertThrows(NullPointerException.class, () -> fields.add((HttpHeader)null, "bogus"));
         assertThat(fields.size(), is(0));
     }
@@ -1067,7 +1065,7 @@ public class HttpFieldsTest
     public void testAddNullValueList()
     {
         HttpFields.Mutable fields = HttpFields.build();
-        assertThrows(NullPointerException.class, () -> fields.add("name", (List<String>)null));
+        fields.add("name", (List<String>)null);
         assertThat(fields.size(), is(0));
         List<String> list = new ArrayList<>();
         fields.add("name", list);
@@ -1462,6 +1460,12 @@ public class HttpFieldsTest
             @Override
             public HttpField onReplaceField(HttpField oldField, HttpField newField)
             {
+                if (newField.getValueList().contains("removeOnReplace"))
+                {
+                    actions.add("onReplaceFieldRemove");
+                    return null;
+                }
+
                 actions.add("onReplaceField");
                 return super.onReplaceField(oldField, newField);
             }
@@ -1488,5 +1492,12 @@ public class HttpFieldsTest
         assertThat(wrapper.size(), is(0));
         assertThat(wrapper.actions, is(List.of("onAddField", "onReplaceField", "onRemoveField")));
         wrapper.actions.clear();
+
+        wrapper.ensureField(new HttpField("ensure", "value0"));
+        wrapper.ensureField(new HttpField("ensure", "value1"));
+        assertThat(wrapper.actions, is(List.of("onAddField", "onReplaceField")));
+        wrapper.ensureField(new HttpField("ensure", "removeOnReplace"));
+        assertThat(wrapper.actions, is(List.of("onAddField", "onReplaceField", "onReplaceFieldRemove")));
+        assertThat(wrapper.getField("ensure"), nullValue());
     }
 }
