@@ -25,6 +25,7 @@ import java.util.concurrent.Executor;
 import org.eclipse.jetty.alpn.client.ALPNClientConnectionFactory;
 import org.eclipse.jetty.http2.BufferingFlowControlStrategy;
 import org.eclipse.jetty.http2.FlowControlStrategy;
+import org.eclipse.jetty.http2.SessionContainer;
 import org.eclipse.jetty.http2.api.Session;
 import org.eclipse.jetty.http2.frames.Frame;
 import org.eclipse.jetty.http2.frames.SettingsFrame;
@@ -101,6 +102,7 @@ import org.eclipse.jetty.util.thread.Scheduler;
 @ManagedObject
 public class HTTP2Client extends ContainerLifeCycle implements AutoCloseable
 {
+    private final SessionContainer container = new SessionContainer();
     private final ClientConnector connector;
     private int inputBufferSize = 8192;
     private List<String> protocols = List.of("h2");
@@ -108,6 +110,7 @@ public class HTTP2Client extends ContainerLifeCycle implements AutoCloseable
     private int initialStreamRecvWindow = 8 * 1024 * 1024;
     private int maxFrameSize = Frame.DEFAULT_MAX_SIZE;
     private int maxConcurrentPushedStreams = 32;
+    private int maxLocalStreams = -1;
     private int maxSettingsKeys = SettingsFrame.DEFAULT_MAX_KEYS;
     private int maxDecoderTableCapacity = HpackContext.DEFAULT_MAX_TABLE_CAPACITY;
     private int maxEncoderTableCapacity = HpackContext.DEFAULT_MAX_TABLE_CAPACITY;
@@ -128,9 +131,15 @@ public class HTTP2Client extends ContainerLifeCycle implements AutoCloseable
     public HTTP2Client(ClientConnector connector)
     {
         this.connector = connector;
+        installBean(container);
         installBean(connector);
     }
 
+    SessionContainer getSessionContainer()
+    {
+        return container;
+    }
+
     public ClientConnector getClientConnector()
     {
         return connector;
@@ -307,6 +316,21 @@ public class HTTP2Client extends ContainerLifeCycle implements AutoCloseable
         this.maxConcurrentPushedStreams = maxConcurrentPushedStreams;
     }
 
+    @ManagedAttribute("The maximum number of concurrent local streams")
+    public int getMaxLocalStreams()
+    {
+        return maxLocalStreams;
+    }
+
+    /**
+     * @param maxLocalStreams the max number of local streams that can be opened,
+     * if the server does not send the max concurrent stream setting.
+     */
+    public void setMaxLocalStreams(int maxLocalStreams)
+    {
+        this.maxLocalStreams = maxLocalStreams;
+    }
+
     @ManagedAttribute("The max number of keys in all SETTINGS frames")
     public int getMaxSettingsKeys()
     {
