@@ -26,6 +26,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.CompletableFuture;
+import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.Exchanger;
 import java.util.concurrent.TimeUnit;
@@ -46,6 +47,7 @@ import org.eclipse.jetty.http2.ErrorCode;
 import org.eclipse.jetty.http2.FlowControlStrategy;
 import org.eclipse.jetty.http2.HTTP2Session;
 import org.eclipse.jetty.http2.HTTP2Stream;
+import org.eclipse.jetty.http2.SessionContainer;
 import org.eclipse.jetty.http2.api.Session;
 import org.eclipse.jetty.http2.api.Stream;
 import org.eclipse.jetty.http2.api.server.ServerSessionListener;
@@ -57,7 +59,6 @@ import org.eclipse.jetty.http2.frames.ResetFrame;
 import org.eclipse.jetty.http2.frames.SettingsFrame;
 import org.eclipse.jetty.http2.frames.WindowUpdateFrame;
 import org.eclipse.jetty.http2.generator.Generator;
-import org.eclipse.jetty.http2.internal.HTTP2Flusher;
 import org.eclipse.jetty.http2.server.AbstractHTTP2ServerConnectionFactory;
 import org.eclipse.jetty.http2.server.HTTP2ServerConnectionFactory;
 import org.eclipse.jetty.io.AbstractEndPoint;
@@ -720,11 +721,10 @@ public class StreamResetTest extends AbstractTest
         Thread.sleep(500);
 
         AbstractHTTP2ServerConnectionFactory http2 = connector.getConnectionFactory(AbstractHTTP2ServerConnectionFactory.class);
-        Set<Session> sessions = http2.getBean(AbstractHTTP2ServerConnectionFactory.HTTP2SessionContainer.class).getSessions();
+        Set<Session> sessions = http2.getBean(SessionContainer.class).getSessions();
         assertEquals(1, sessions.size());
         HTTP2Session session = (HTTP2Session)sessions.iterator().next();
-        HTTP2Flusher flusher = session.getBean(HTTP2Flusher.class);
-        assertEquals(0, flusher.getFrameQueueSize());
+        assertEquals(0, session.getFrameQueueSize());
     }
 
     @Test
@@ -1123,6 +1123,67 @@ public class StreamResetTest extends AbstractTest
         await().atMost(5, TimeUnit.SECONDS).until(() -> ((HTTP2Session)stream.getSession()).updateSendWindow(0), greaterThan(0));
     }
 
+    @Test
+    public void testClientResetThenDoesNotResetInFlightFrames() throws Exception
+    {
+        CountDownLatch clientResetLatch = new CountDownLatch(1);
+        start(new ServerSessionListener()
+        {
+            @Override
+            public Stream.Listener onNewStream(Stream stream, HeadersFrame frame)
+            {
+                try
+                {
+                    if (clientResetLatch.await(5, TimeUnit.SECONDS))
+                    {
+                        MetaData.Response response = new MetaData.Response(200, null, HttpVersion.HTTP_2, HttpFields.EMPTY);
+                        HeadersFrame responseFrame = new HeadersFrame(stream.getId(), response, null, false);
+                        stream.headers(responseFrame)
+                            .thenCompose(s -> s.data(new DataFrame(s.getId(), ByteBuffer.allocate(128), false)))
+                            .thenCompose(s -> s.data(new DataFrame(s.getId(), ByteBuffer.allocate(64), true)));
+                    }
+                    return null;
+                }
+                catch (InterruptedException x)
+                {
+                    throw new RuntimeException(x);
+                }
+            }
+        });
+
+        List<Frame> inFrames = new CopyOnWriteArrayList<>();
+        List<Frame> outFrames = new CopyOnWriteArrayList<>();
+        http2Client.addBean(new HTTP2Session.FrameListener()
+        {
+            @Override
+            public void onIncomingFrame(Session session, Frame frame)
+            {
+                inFrames.add(frame);
+            }
+
+            @Override
+            public void onOutgoingFrame(Session session, Frame frame)
+            {
+                outFrames.add(frame);
+            }
+        });
+
+        Session client = newClientSession(new Session.Listener() {});
+        MetaData.Request request = newRequest("GET", HttpFields.EMPTY);
+        HeadersFrame requestFrame = new HeadersFrame(request, null, true);
+        client.newStream(requestFrame, new Stream.Listener() {})
+            .thenCompose(s -> s.reset(new ResetFrame(s.getId(), ErrorCode.ENHANCE_YOUR_CALM_ERROR.code)))
+            .thenRun(clientResetLatch::countDown);
+
+        // Wait until received all the frames from the server.
+        await().atMost(5, TimeUnit.SECONDS).untilAsserted(() ->
+            assertTrue(inFrames.stream().anyMatch(f -> f instanceof DataFrame d && d.isEndStream()), inFrames.toString()));
+
+        // Verify that we only sent 1 RST_STREAM frame.
+        await().during(1, TimeUnit.SECONDS).atMost(5, TimeUnit.SECONDS).untilAsserted(() ->
+            assertEquals(1L, outFrames.stream().filter(f -> f instanceof ResetFrame).count(), outFrames.toString()));
+    }
+
     private void waitUntilTCPCongested(WriteFlusher flusher) throws TimeoutException, InterruptedException
     {
         long start = NanoTime.now();
