@@ -24,6 +24,7 @@ import java.util.concurrent.atomic.AtomicInteger;
 import java.util.function.LongSupplier;
 
 import org.eclipse.jetty.http.HttpField;
+import org.eclipse.jetty.http.HttpTokens;
 import org.eclipse.jetty.http.MetaData;
 import org.eclipse.jetty.http.compression.NBitIntegerDecoder;
 import org.eclipse.jetty.http3.qpack.internal.QpackContext;
@@ -40,7 +41,6 @@ import org.eclipse.jetty.util.component.Dumpable;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import static org.eclipse.jetty.http3.qpack.QpackException.H3_GENERAL_PROTOCOL_ERROR;
 import static org.eclipse.jetty.http3.qpack.QpackException.QPACK_DECOMPRESSION_FAILED;
 import static org.eclipse.jetty.http3.qpack.QpackException.QPACK_ENCODER_STREAM_ERROR;
 
@@ -208,9 +208,9 @@ public class QpackDecoder implements Dumpable
             notifyMetaDataHandler(false);
             return hadMetaData;
         }
-        catch (QpackException.SessionException e)
+        catch (QpackException x)
         {
-            throw e;
+            throw x;
         }
         catch (Throwable t)
         {
@@ -368,7 +368,7 @@ public class QpackDecoder implements Dumpable
         public void onSetDynamicTableCapacity(int capacity) throws QpackException
         {
             if (capacity > getMaxTableCapacity())
-                throw new QpackException.StreamException(H3_GENERAL_PROTOCOL_ERROR, "DynamicTable capacity exceeds max capacity");
+                throw new QpackException.SessionException(QPACK_ENCODER_STREAM_ERROR, "DynamicTable capacity exceeds max capacity");
             _context.getDynamicTable().setCapacity(capacity);
         }
 
@@ -398,6 +398,10 @@ public class QpackDecoder implements Dumpable
             DynamicTable dynamicTable = _context.getDynamicTable();
             Entry referencedEntry = isDynamicTableIndex ? dynamicTable.getRelative(nameIndex) : staticTable.get(nameIndex);
 
+            // Verify the field value before inserting into the table.
+            if (!HttpTokens.isLegalFieldValue(value))
+                throw new QpackException.SessionException(QPACK_ENCODER_STREAM_ERROR, "Invalid header value");
+
             // Add the new Entry to the DynamicTable.
             Entry entry = new Entry(new HttpField(referencedEntry.getHttpField().getHeader(), referencedEntry.getHttpField().getName(), value));
             dynamicTable.add(entry);
@@ -413,6 +417,12 @@ public class QpackDecoder implements Dumpable
 
             Entry entry = new Entry(new HttpField(name, value));
 
+            // Verify the field name and value before inserting into the table.
+            if (!HttpTokens.isLegalH2H3FieldName(name))
+                throw new QpackException.SessionException(QPACK_ENCODER_STREAM_ERROR, "Invalid header name: " + name);
+            if (!HttpTokens.isLegalFieldValue(value))
+                throw new QpackException.SessionException(QPACK_ENCODER_STREAM_ERROR, "Invalid header value: " + value);
+
             // Add the new Entry to the DynamicTable.
             DynamicTable dynamicTable = _context.getDynamicTable();
             dynamicTable.add(entry);
