@@ -20,10 +20,14 @@ import java.util.concurrent.TimeoutException;
 import java.util.function.BiConsumer;
 import java.util.function.Supplier;
 
+import org.eclipse.jetty.http.BadMessageException;
 import org.eclipse.jetty.http.ComplianceViolation;
 import org.eclipse.jetty.http.HttpCompliance;
 import org.eclipse.jetty.http.HttpException;
+import org.eclipse.jetty.http.HttpField;
 import org.eclipse.jetty.http.HttpFields;
+import org.eclipse.jetty.http.HttpHeader;
+import org.eclipse.jetty.http.HttpHeaderValue;
 import org.eclipse.jetty.http.HttpMethod;
 import org.eclipse.jetty.http.HttpStatus;
 import org.eclipse.jetty.http.HttpVersion;
@@ -104,6 +108,10 @@ public class HttpStreamOverHTTP3 implements HttpStream
                     System.lineSeparator(), fields);
             }
 
+            HttpField expectField = fields.getField(HttpHeader.EXPECT);
+            if (expectField != null && !HttpHeaderValue.CONTINUE.is(expectField.getValue()))
+                throw new BadMessageException(HttpStatus.EXPECTATION_FAILED_417);
+
             InvocationType invocationType = Invocable.getInvocationType(handler);
             return new ReadyTask(invocationType, handler)
             {
@@ -185,7 +193,11 @@ public class HttpStreamOverHTTP3 implements HttpStream
         {
             Runnable task = httpChannel.onContentAvailable();
             if (task != null)
-                connection.offer(task);
+            {
+                // We must dispatch, so an application thread does not
+                // become a producer and then consume another request.
+                connection.offer(task, true);
+            }
         }
         else
         {
@@ -398,7 +410,7 @@ public class HttpStreamOverHTTP3 implements HttpStream
                 {
                     if (hasContent)
                     {
-                        callback.completeWith(sendDataFrame(content, lastContent, false)
+                        callback.completeWith(sendDataFrame(content, false, false)
                             .thenCompose(s -> sendTrailerFrame(trailers)));
                     }
                     else
