@@ -22,6 +22,7 @@ import java.time.Instant;
 import java.util.Arrays;
 import java.util.HashMap;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
 import java.util.Objects;
 import java.util.Set;
@@ -30,6 +31,7 @@ import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.concurrent.ThreadLocalRandom;
 import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.concurrent.atomic.AtomicLong;
 import java.util.concurrent.atomic.LongAdder;
 import java.util.function.IntUnaryOperator;
 import java.util.function.ToLongFunction;
@@ -73,8 +75,8 @@ public class ArrayByteBufferPool implements ByteBufferPool, Dumpable
     private final IntUnaryOperator _bucketIndexFor;
     private final IntUnaryOperator _bucketCapacity;
     private final AtomicBoolean _evictor = new AtomicBoolean(false);
-    private final ConcurrentMap<Integer, Long> _noBucketDirectAcquires = new ConcurrentHashMap<>();
-    private final ConcurrentMap<Integer, Long> _noBucketIndirectAcquires = new ConcurrentHashMap<>();
+    private final ConcurrentHashMap<Integer, NoBucketData> _noBucketDirectAcquires = new ConcurrentHashMap<>();
+    private final ConcurrentHashMap<Integer, NoBucketData> _noBucketIndirectAcquires = new ConcurrentHashMap<>();
     private boolean _statisticsEnabled;
 
     /**
@@ -238,18 +240,24 @@ public class ArrayByteBufferPool implements ByteBufferPool, Dumpable
 
     private void recordNoBucketAcquire(int size, boolean direct)
     {
+        ConcurrentMap<Integer, NoBucketData> map = direct ? _noBucketDirectAcquires : _noBucketIndirectAcquires;
+        int key;
         if (isStatisticsEnabled())
         {
-            ConcurrentMap<Integer, Long> map = direct ? _noBucketDirectAcquires : _noBucketIndirectAcquires;
             int idx = _bucketIndexFor.applyAsInt(size);
-            int key = _bucketCapacity.applyAsInt(idx);
-            map.compute(key, (k, v) ->
-            {
-                if (v == null)
-                    return 1L;
-                return v + 1L;
-            });
+            key = _bucketCapacity.applyAsInt(idx);
         }
+        else
+        {
+            key = 0;
+        }
+        map.compute(key, (k, v) ->
+        {
+            if (v == null)
+                return new NoBucketData(1L, isStatisticsEnabled() ? new Throwable("Acquired by " + Thread.currentThread().getName()) : null);
+            v.counter.incrementAndGet();
+            return v;
+        });
     }
 
     @Override
@@ -257,7 +265,9 @@ public class ArrayByteBufferPool implements ByteBufferPool, Dumpable
     {
         RetainableByteBuffer actual = buffer;
         while (actual instanceof RetainableByteBuffer.Wrapper wrapper)
+        {
             actual = wrapper.getWrapped();
+        }
 
         if (actual instanceof ReservedBuffer reservedBuffer)
         {
@@ -306,6 +316,9 @@ public class ArrayByteBufferPool implements ByteBufferPool, Dumpable
     {
         bucket.recordRelease();
 
+        if (entry.isTerminated())
+            return;
+
         RetainableByteBuffer buffer = entry.getPooled();
         BufferUtil.reset(buffer.getByteBuffer());
 
@@ -314,7 +327,7 @@ public class ArrayByteBufferPool implements ByteBufferPool, Dumpable
         if (entry.release())
         {
             if (used % 100 == 0)
-               checkMaxMemory(bucket, buffer.isDirect());
+                checkMaxMemory(bucket, buffer.isDirect());
             return;
         }
 
@@ -443,7 +456,9 @@ public class ArrayByteBufferPool implements ByteBufferPool, Dumpable
     {
         long size = 0;
         for (RetainedBucket bucket : direct ? _direct : _indirect)
+        {
             size += count.applyAsLong(bucket) * bucket.getCapacity();
+        }
         return size;
     }
 
@@ -496,7 +511,8 @@ public class ArrayByteBufferPool implements ByteBufferPool, Dumpable
 
     private Map<Integer, Long> getNoBucketAcquires(boolean direct)
     {
-        return new HashMap<>(direct ? _noBucketDirectAcquires : _noBucketIndirectAcquires);
+        ConcurrentMap<Integer, NoBucketData> map = direct ? _noBucketDirectAcquires : _noBucketIndirectAcquires;
+        return map.entrySet().stream().collect(Collectors.toMap(Map.Entry::getKey, e -> e.getValue().counter.get()));
     }
 
     @ManagedOperation(value = "Clears this ByteBufferPool", impact = "ACTION")
@@ -541,7 +557,30 @@ public class ArrayByteBufferPool implements ByteBufferPool, Dumpable
             getDirectMemory(), _maxDirectMemory);
     }
 
-    private class RetainedBucket
+    private static class NoBucketData
+    {
+        private final AtomicLong counter;
+        private final Throwable acquireStack;
+
+        private NoBucketData(long counter, Throwable acquireStack)
+        {
+            this.counter = new AtomicLong(counter);
+            this.acquireStack = acquireStack;
+        }
+
+        @Override
+        public String toString()
+        {
+            if (acquireStack == null)
+                return Long.toString(counter.get());
+            StringWriter w = new StringWriter();
+            PrintWriter pw = new PrintWriter(w);
+            acquireStack.printStackTrace(pw);
+            return counter + " from " + w;
+        }
+    }
+
+    private class RetainedBucket implements Dumpable
     {
         private final LongAdder _acquires = new LongAdder();
         private final LongAdder _pooled = new LongAdder();
@@ -647,6 +686,12 @@ public class ArrayByteBufferPool implements ByteBufferPool, Dumpable
             getPool().stream().forEach(Pool.Entry::remove);
         }
 
+        @Override
+        public void dump(Appendable out, String indent) throws IOException
+        {
+            Dumpable.dumpObjects(out, indent, this, _pool);
+        }
+
         @Override
         public String toString()
         {
@@ -676,7 +721,7 @@ public class ArrayByteBufferPool implements ByteBufferPool, Dumpable
             @Override
             public String toString()
             {
-                return "capacity=%d,in-use=%d/%d,pooled/acquires/releases=%d/%d/%d(%.3f%%),non-pooled/evicts/removes=%d/%d/%d".formatted(
+                return String.format(Locale.ROOT, "capacity=%d,in-use=%d/%d,pooled/acquires/releases=%d/%d/%d(%.3f%%),non-pooled/evicts/removes=%d/%d/%d",
                     capacity,
                     inUseEntries,
                     totalEntries,
@@ -853,6 +898,21 @@ public class ArrayByteBufferPool implements ByteBufferPool, Dumpable
                 .collect(Collectors.joining(System.lineSeparator()));
         }
 
+        @Override
+        public void dump(Appendable out, String indent) throws IOException
+        {
+            Dumpable.dumpObjects(
+                out,
+                indent,
+                this,
+                DumpableCollection.fromArray("direct", ((ArrayByteBufferPool)this)._direct),
+                new DumpableMap("direct non-pooled acquisitions", ((ArrayByteBufferPool)this)._noBucketDirectAcquires),
+                DumpableCollection.fromArray("indirect", ((ArrayByteBufferPool)this)._indirect),
+                new DumpableMap("heap non-pooled acquisitions", ((ArrayByteBufferPool)this)._noBucketIndirectAcquires),
+                DumpableCollection.from("leaks", getLeaks().stream().map(Buffer::dump).toList())
+            );
+        }
+
         public class Buffer extends RetainableByteBuffer.Wrapper
         {
             private final int size;
@@ -867,7 +927,7 @@ public class ArrayByteBufferPool implements ByteBufferPool, Dumpable
                 super(wrapped);
                 this.size = size;
                 this.acquireInstant = Instant.now();
-                this.acquireStack = new Throwable();
+                this.acquireStack = new Throwable("Acquired by " + Thread.currentThread().getName());
             }
 
             public int getSize()
@@ -889,7 +949,7 @@ public class ArrayByteBufferPool implements ByteBufferPool, Dumpable
             public void retain()
             {
                 super.retain();
-                retainStacks.add(new Throwable());
+                retainStacks.add(new Throwable("Retained by " + Thread.currentThread().getName()));
             }
 
             @Override
@@ -904,14 +964,16 @@ public class ArrayByteBufferPool implements ByteBufferPool, Dumpable
                         if (LOG.isDebugEnabled())
                             LOG.debug("released {}", this);
                     }
-                    releaseStacks.add(new Throwable());
+                    releaseStacks.add(new Throwable("Released by " + Thread.currentThread().getName()));
                     return released;
                 }
                 catch (IllegalStateException e)
                 {
                     buffers.add(this);
-                    overReleaseStacks.add(new Throwable());
-                    throw e;
+                    overReleaseStacks.add(new Throwable("Over-released by " + Thread.currentThread().getName()));
+                    IllegalStateException ise = new IllegalStateException(Thread.currentThread().getName() + " over-released " + this);
+                    releaseStacks.forEach(ise::addSuppressed);
+                    throw ise;
                 }
             }
 
@@ -935,7 +997,13 @@ public class ArrayByteBufferPool implements ByteBufferPool, Dumpable
                 {
                     overReleaseStack.printStackTrace(pw);
                 }
-                return "%s@%x of %d bytes on %s wrapping %s acquired at %s".formatted(getClass().getSimpleName(), hashCode(), getSize(), getAcquireInstant(), getWrapped(), w);
+                String stacks = w.toString();
+                return ("%s@%x of %d bytes on %s wrapping %s%n" +
+                    " %s%n" +
+                    " acquired at %s")
+                    .formatted(getClass().getSimpleName(), hashCode(), getSize(), getAcquireInstant(), getWrapped(),
+                        BufferUtil.toDetailString(getByteBuffer()),
+                        stacks);
             }
         }
     }
