@@ -13,6 +13,7 @@
 
 package org.eclipse.jetty.io;
 
+import java.nio.ByteBuffer;
 import java.nio.ByteOrder;
 import java.util.ArrayList;
 import java.util.Collections;
@@ -20,10 +21,13 @@ import java.util.List;
 import java.util.Map;
 
 import org.eclipse.jetty.io.internal.CompoundPool;
+import org.eclipse.jetty.util.BufferUtil;
 import org.eclipse.jetty.util.ConcurrentPool;
 import org.eclipse.jetty.util.Pool;
 import org.hamcrest.Matchers;
 import org.junit.jupiter.api.Test;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.ValueSource;
 
 import static org.hamcrest.MatcherAssert.assertThat;
 import static org.hamcrest.Matchers.containsString;
@@ -42,10 +46,49 @@ import static org.junit.jupiter.api.Assertions.assertTrue;
 public class ArrayByteBufferPoolTest
 {
     @Test
-    public void testDump()
+    public void testRemoveAndReleaseFromCompoundPool()
+    {
+        int bufferCount = ConcurrentPool.OPTIMAL_MAX_SIZE * 2;
+        List<RetainableByteBuffer> rbbs = new ArrayList<>();
+
+        ArrayByteBufferPool pool = new ArrayByteBufferPool();
+        pool.setStatisticsEnabled(true);
+        for (int i = 0; i < bufferCount; i++)
+        {
+            RetainableByteBuffer rbb = pool.acquire(1, false);
+            rbbs.add(rbb);
+        }
+        rbbs.forEach(Retainable::release);
+        rbbs.clear();
+
+        for (int i = 0; i < bufferCount; i++)
+        {
+            RetainableByteBuffer rbb = pool.acquire(1, false);
+            ByteBuffer byteBuffer = rbb.getByteBuffer();
+            BufferUtil.flipToFill(byteBuffer);
+            byteBuffer.position(1);
+            rbbs.add(rbb);
+        }
+        rbbs.forEach(pool::removeAndRelease);
+
+        for (RetainableByteBuffer rbb : rbbs)
+        {
+            assertThat(rbb.getByteBuffer(), notNullValue());
+            assertEquals(1, rbb.getByteBuffer().position());
+        }
+
+        assertEquals(0L, pool.getAvailableHeapByteBufferCount());
+
+        String dump = pool.dump();
+        assertThat(dump, containsString("[capacity=4096,in-use=0/0,pooled/acquires/releases=512/1024/1024(50.000%),non-pooled/evicts/removes=0/0/512]"));
+    }
+
+    @ParameterizedTest
+    @ValueSource(booleans = {true, false})
+    public void testDump(boolean statsEnabled)
     {
         ArrayByteBufferPool pool = new ArrayByteBufferPool(0, 10, 100, Integer.MAX_VALUE, 200, 200);
-        pool.setStatisticsEnabled(true);
+        pool.setStatisticsEnabled(statsEnabled);
 
         List<RetainableByteBuffer> buffers = new ArrayList<>();
 
@@ -55,12 +98,20 @@ public class ArrayByteBufferPoolTest
         buffers.forEach(RetainableByteBuffer::release);
 
         String dump = pool.dump();
-        assertThat(dump, containsString("direct non-pooled acquisitions size=5\n"));
-        assertThat(dump, containsString("110: 10\n"));
-        assertThat(dump, containsString("120: 10\n"));
-        assertThat(dump, containsString("130: 10\n"));
-        assertThat(dump, containsString("140: 10\n"));
-        assertThat(dump, containsString("150: 10\n"));
+        if (statsEnabled)
+        {
+            assertThat(dump, containsString("direct non-pooled acquisitions size=5\n"));
+            assertThat(dump, containsString("110: 10 from "));
+            assertThat(dump, containsString("120: 10 from "));
+            assertThat(dump, containsString("130: 10 from "));
+            assertThat(dump, containsString("140: 10 from "));
+            assertThat(dump, containsString("150: 10 from "));
+        }
+        else
+        {
+            assertThat(dump, containsString("direct non-pooled acquisitions size=1\n"));
+            assertThat(dump, containsString("0: 50\n"));
+        }
         pool.clear();
         assertThat(pool.dump(), containsString("direct non-pooled acquisitions size=0\n"));
     }
