@@ -36,6 +36,7 @@ import java.util.function.Consumer;
 import java.util.function.Function;
 import java.util.function.Predicate;
 
+import org.eclipse.jetty.http.BadMessageException;
 import org.eclipse.jetty.http.ComplianceViolation;
 import org.eclipse.jetty.http.HttpCookie;
 import org.eclipse.jetty.http.HttpFields;
@@ -271,6 +272,7 @@ public interface Request extends Attributes, Content.Source
      * there is no content currently available, or it reaches EOF.
      * The {@link HttpConfiguration#setMaxUnconsumedRequestContentReads(int)} configuration can be used
      * to configure how many reads will be attempted by this method.
+     * If the content was not fully consumed, the underlying connection may be made non-persistent.
      * @return true if the content was fully consumed.
      */
     boolean consumeAvailable();
@@ -513,25 +515,27 @@ public interface Request extends Attributes, Content.Source
         if (fields == null)
             return DEFAULT_LOCALES;
 
-        List<String> acceptable = fields.getQualityCSV(HttpHeader.ACCEPT_LANGUAGE);
+        List<String> acceptable =
+            fields.getQualityCSV(HttpHeader.ACCEPT_LANGUAGE)
+                .stream()
+                .filter(StringUtil::isNotBlank)
+                .toList();
 
-        // return sorted list of locals, with known locales in quality order before unknown locales in quality order
-        return switch (acceptable.size())
-        {
-            case 0 -> DEFAULT_LOCALES;
-            case 1 -> List.of(Locale.forLanguageTag(acceptable.get(0)));
-            default ->
-            {
-                List<Locale> locales = acceptable.stream().map(Locale::forLanguageTag).toList();
-                List<Locale> known = locales.stream().filter(MimeTypes::isKnownLocale).toList();
-                if (known.size() == locales.size())
-                    yield locales; // All locales are known
-                List<Locale> unknown = locales.stream().filter(l -> !MimeTypes.isKnownLocale(l)).toList();
-                locales = new ArrayList<>(known);
-                locales.addAll(unknown);
-                yield locales; // List of known locales before unknown locales
-            }
-        };
+        if (acceptable.isEmpty())
+            return DEFAULT_LOCALES;
+
+        // return sorted list of locales, with known locales in quality order before unknown locales in quality order
+        List<Locale> locales = acceptable.stream().map(Locale::forLanguageTag).toList();
+        // Filter again, only allowing known locales
+        List<Locale> known = locales.stream().filter(MimeTypes::isKnownLocale).toList();
+        if (known.isEmpty())
+            return DEFAULT_LOCALES;
+        if (known.size() == locales.size())
+            return locales; // All locales are known
+        List<Locale> unknown = locales.stream().filter(l -> !MimeTypes.isKnownLocale(l)).toList();
+        locales = new ArrayList<>(known);
+        locales.addAll(unknown);
+        return locales; // List of known locales before unknown locales
     }
 
     static InputStream asInputStream(Request request)
@@ -559,27 +563,37 @@ public interface Request extends Attributes, Content.Source
 
     static Fields extractQueryParameters(Request request, Charset charset)
     {
-        String query = request.getHttpURI().getQuery();
-        if (StringUtil.isBlank(query))
-            return Fields.EMPTY;
-        Fields fields = new Fields(true);
+        UriCompliance uriCompliance = null;
+        try
+        {
+            String query = request.getHttpURI().getQuery();
+            if (StringUtil.isBlank(query))
+                return Fields.EMPTY;
+            Fields fields = new Fields(true);
 
-        if (charset == null || StandardCharsets.UTF_8.equals(charset))
-        {
-            UriCompliance uriCompliance = request.getConnectionMetaData().getHttpConfiguration().getUriCompliance();
-            boolean allowBadUtf8 = uriCompliance.allows(UriCompliance.Violation.BAD_UTF8_ENCODING);
-            if (!UrlEncoded.decodeUtf8To(query, 0, query.length(), fields::add, allowBadUtf8))
+            if (charset == null || StandardCharsets.UTF_8.equals(charset))
             {
-                HttpChannel httpChannel = HttpChannel.from(request);
-                if (httpChannel != null && httpChannel.getComplianceViolationListener() != null)
-                    httpChannel.getComplianceViolationListener().onComplianceViolation(new ComplianceViolation.Event(uriCompliance, UriCompliance.Violation.BAD_UTF8_ENCODING, "query=" + query));
+                uriCompliance = request.getConnectionMetaData().getHttpConfiguration().getUriCompliance();
+                boolean allowBadPercent = uriCompliance.allows(UriCompliance.Violation.BAD_PERCENT_ENCODING);
+                boolean allowBadUtf8 = uriCompliance.allows(UriCompliance.Violation.BAD_UTF8_ENCODING);
+                boolean allowTruncatedUtf8 = uriCompliance.allows(UriCompliance.Violation.TRUNCATED_UTF8_ENCODING);
+                if (!UrlEncoded.decodeUtf8To(query, 0, query.length(), fields::add, allowBadPercent, allowBadUtf8, allowTruncatedUtf8))
+                {
+                    HttpChannel httpChannel = HttpChannel.from(request);
+                    if (httpChannel != null && httpChannel.getComplianceViolationListener() != null)
+                        httpChannel.getComplianceViolationListener().onComplianceViolation(new ComplianceViolation.Event(uriCompliance, UriCompliance.Violation.BAD_UTF8_ENCODING, "query=" + query));
+                }
             }
+            else
+            {
+                UrlEncoded.decodeTo(query, fields::add, charset);
+            }
+            return fields;
         }
-        else
+        catch (Throwable t)
         {
-            UrlEncoded.decodeTo(query, fields::add, charset);
+            throw new BadMessageException("Bad query", t);
         }
-        return fields;
     }
 
     static Fields getParameters(Request request) throws Exception
@@ -1010,6 +1024,12 @@ public interface Request extends Attributes, Content.Source
         }
     }
 
+    /** Unwrap a Request back to the given type.
+     * @param request the possibly wrapped request to unwrap
+     * @param type the type to unwrap to
+     * @return the request unwrapped back to the given type or
+     * null if it cannot be unwrapped to the supplied type.
+     */
     @SuppressWarnings("unchecked")
     static <T> T as(Request request, Class<T> type)
     {
@@ -1022,6 +1042,34 @@ public interface Request extends Attributes, Content.Source
         return null;
     }
 
+    /**
+     * Unwrap a Request back to the given type, ensuring that we do not cross a
+     * context boundary (as might be the case during cross-context dispatch).
+     *
+     * @param request the possibly wrapped request to unwrap
+     * @param type the type to unwrap to
+     * @return the request unwrapped back to the given type, or null if it cannot be
+     * unwrapped to that type or a context boundary is crossed.
+     */
+    static <T extends Request> T asInContext(Request request, Class<T> type)
+    {
+        //the context whose boundary should not be crossed
+        Context context = request == null ? null : request.getContext();
+
+        if (context == null)
+            return Request.as(request, type);
+
+        while (request != null)
+        {
+            if (request.getContext() != context)
+                return null;
+            if (type.isInstance(request))
+                return (T)request;
+            request = request instanceof Request.Wrapper wrapper ? wrapper.getWrapped() : null;
+        }
+        return null;
+    }
+
     @SuppressWarnings("unchecked")
     static <T, R> R get(Request request, Class<T> type, Function<T, R> getter)
     {
