@@ -28,6 +28,7 @@ import java.util.ListIterator;
 import java.util.Objects;
 import java.util.Set;
 import java.util.concurrent.CopyOnWriteArrayList;
+import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.function.Consumer;
 import java.util.function.Predicate;
@@ -74,7 +75,7 @@ import org.slf4j.LoggerFactory;
 public class ContextHandler extends Handler.Wrapper implements Attributes, AliasCheck
 {
     private static final Logger LOG = LoggerFactory.getLogger(ContextHandler.class);
-    private static final ThreadLocal<Context> __context = new ThreadLocal<>();
+    private static final ThreadLocal<Context> CURRENT_CONTEXT = new ThreadLocal<>();
 
     public static final String MANAGED_ATTRIBUTES = "org.eclipse.jetty.server.context.ManagedAttributes";
 
@@ -92,7 +93,7 @@ public class ContextHandler extends Handler.Wrapper implements Attributes, Alias
      */
     public static Context getCurrentContext()
     {
-        return __context.get();
+        return CURRENT_CONTEXT.get();
     }
 
     /**
@@ -103,7 +104,7 @@ public class ContextHandler extends Handler.Wrapper implements Attributes, Alias
      */
     public static Context getCurrentContext(Server server)
     {
-        Context context = __context.get();
+        Context context = CURRENT_CONTEXT.get();
         return context == null ? (server == null ? null : server.getContext()) : context;
     }
 
@@ -116,7 +117,7 @@ public class ContextHandler extends Handler.Wrapper implements Attributes, Alias
 
     public static ContextHandler getContextHandler(Request request)
     {
-        ContextRequest contextRequest = Request.as(request, ContextRequest.class);
+        ContextRequest contextRequest = Request.asInContext(request, ContextRequest.class);
         if (contextRequest == null)
             return null;
         return contextRequest.getContext() instanceof ScopedContext scoped ? scoped.getContextHandler() : null;
@@ -133,6 +134,7 @@ public class ContextHandler extends Handler.Wrapper implements Attributes, Alias
     private final MimeTypes.Wrapper _mimeTypes = new MimeTypes.Wrapper();
     private final List<ContextScopeListener> _contextListeners = new CopyOnWriteArrayList<>();
     private final List<VHost> _vhosts = new ArrayList<>();
+    private final AtomicBoolean _enterScopeSetClassloaderFailed = new AtomicBoolean();
 
     private String _displayName;
     private String _contextPath = "/";
@@ -450,6 +452,11 @@ public class ContextHandler extends Handler.Wrapper implements Attributes, Alias
         return _classLoader;
     }
 
+    /**
+     * The {@link ClassLoader} to set as the {@link Thread#setContextClassLoader(ClassLoader) thread's context classloader}
+     * when the thread executing this handler enters the scope of this context.
+     * @param contextLoader the {@link ClassLoader} or {@code null} to avoid changing the thread's context classloader.
+     */
     public void setClassLoader(ClassLoader contextLoader)
     {
         if (isStarted())
@@ -592,8 +599,8 @@ public class ContextHandler extends Handler.Wrapper implements Attributes, Alias
             if (listener instanceof ContextScopeListener contextScopeListener)
             {
                 _contextListeners.add(contextScopeListener);
-                if (__context.get() != null)
-                    contextScopeListener.enterScope(__context.get(), null);
+                if (CURRENT_CONTEXT.get() != null)
+                    contextScopeListener.enterScope(CURRENT_CONTEXT.get(), null);
             }
             return true;
         }
@@ -608,8 +615,8 @@ public class ContextHandler extends Handler.Wrapper implements Attributes, Alias
             if (listener instanceof ContextScopeListener contextScopeListener)
             {
                 _contextListeners.remove(contextScopeListener);
-                if (__context.get() != null)
-                    contextScopeListener.exitScope(__context.get(), null);
+                if (CURRENT_CONTEXT.get() != null)
+                    contextScopeListener.exitScope(CURRENT_CONTEXT.get(), null);
             }
             return true;
         }
@@ -618,10 +625,25 @@ public class ContextHandler extends Handler.Wrapper implements Attributes, Alias
 
     protected ClassLoader enterScope(Request contextRequest)
     {
+        CURRENT_CONTEXT.set(_context);
+
         ClassLoader lastLoader = Thread.currentThread().getContextClassLoader();
-        __context.set(_context);
-        if (_classLoader != null)
-            Thread.currentThread().setContextClassLoader(_classLoader);
+        if (_classLoader != null && _classLoader != lastLoader)
+        {
+            try
+            {
+                Thread.currentThread().setContextClassLoader(_classLoader);
+            }
+            catch (Throwable x)
+            {
+                // Log as warning the first time it happens.
+                String msg = "Error setting a context classloader on thread {}";
+                if (_enterScopeSetClassloaderFailed.compareAndSet(false, true))
+                    LOG.warn(msg, Thread.currentThread(), x);
+                else if (LOG.isDebugEnabled())
+                    LOG.debug(msg, Thread.currentThread(), x);
+            }
+        }
         notifyEnterScope(contextRequest);
         return lastLoader;
     }
@@ -644,11 +666,20 @@ public class ContextHandler extends Handler.Wrapper implements Attributes, Alias
         }
     }
 
-    protected void exitScope(Request request, Context lastContext, ClassLoader lastLoader)
+    /**
+     * <p>Exits the scope of the {@link Context}.</p>
+     *
+     * @param contextRequest the context's {@link Request}
+     * @param lastContext the previous context to restore as the current one.
+     * @param lastLoader the previous {@link Thread#getContextClassLoader() thread's context classloader} to restore
+     */
+    protected void exitScope(Request contextRequest, Context lastContext, ClassLoader lastLoader)
     {
-        notifyExitScope(request);
-        __context.set(lastContext);
-        Thread.currentThread().setContextClassLoader(lastLoader);
+        CURRENT_CONTEXT.set(lastContext);
+
+        notifyExitScope(contextRequest);
+        if (Thread.currentThread().getContextClassLoader() != lastLoader)
+            Thread.currentThread().setContextClassLoader(lastLoader);
     }
 
     /**
@@ -743,10 +774,15 @@ public class ContextHandler extends Handler.Wrapper implements Attributes, Alias
             {
                 URI realUri = baseResource.getRealURI();
                 if (realUri == null)
-                    LOG.warn("Base Resource should not be an alias (100% of requests to context are subject to Security/Alias Checks): {}", baseResource);
+                {
+                    LOG.warn("{} Base Resource should not be an alias (100% of requests to context are subject to Security/Alias Checks): {}", getDisplayName(), baseResource);
+                }
                 else
-                    LOG.warn("Base Resource should not be an alias (100% of requests to context are subject to Security/Alias Checks): {} points to {}",
-                        baseResource, realUri.toASCIIString());
+                {
+                    LOG.info("{} Base Resource is an alias: {} -> {}", getDisplayName(), baseResource, realUri.toASCIIString());
+                    setAttribute("_baseResource", _baseResource);
+                    _baseResource = ResourceFactory.of(this).newResource(realUri);
+                }
             }
         }
 
@@ -795,7 +831,7 @@ public class ContextHandler extends Handler.Wrapper implements Attributes, Alias
 
             // is it usable
             if (!tempDirectory.canWrite() || !tempDirectory.isDirectory())
-                throw new IllegalArgumentException("Temp dir " + tempDirectory + " not useable: writeable=" + tempDirectory.canWrite() + ", dir=" + tempDirectory.isDirectory());
+                throw new IllegalArgumentException("Temp dir " + tempDirectory + " not usable: writeable=" + tempDirectory.canWrite() + ", dir=" + tempDirectory.isDirectory());
         }
     }
 
@@ -805,6 +841,8 @@ public class ContextHandler extends Handler.Wrapper implements Attributes, Alias
         _context.call(super::doStop, null);
         cleanupAfterStop();
         _tempDirectoryCreated = false;
+        if (removeAttribute("_baseResource") instanceof Resource baseResource)
+            _baseResource = baseResource;
     }
 
     protected void cleanupAfterStop() throws Exception
@@ -1040,7 +1078,7 @@ public class ContextHandler extends Handler.Wrapper implements Attributes, Alias
             return true;
         }
 
-        if (pathInContext.length() == 0 && !getAllowNullPathInContext())
+        if (pathInContext.isEmpty() && !getAllowNullPathInContext())
         {
             handleMovedPermanently(request, response, callback);
             return true;
@@ -1445,7 +1483,7 @@ public class ContextHandler extends Handler.Wrapper implements Attributes, Alias
 
         public void call(Invocable.Callable callable, Request request) throws Exception
         {
-            Context lastContext = __context.get();
+            Context lastContext = CURRENT_CONTEXT.get();
             if (lastContext == this)
                 callable.call();
             else
@@ -1464,7 +1502,7 @@ public class ContextHandler extends Handler.Wrapper implements Attributes, Alias
 
         public <T> boolean test(Predicate<T> predicate, T t, Request request)
         {
-            Context lastContext = __context.get();
+            Context lastContext = CURRENT_CONTEXT.get();
             if (lastContext == this)
                 return predicate.test(t);
 
@@ -1481,7 +1519,7 @@ public class ContextHandler extends Handler.Wrapper implements Attributes, Alias
 
         public void accept(Consumer<Throwable> consumer, Throwable t, Request request)
         {
-            Context lastContext = __context.get();
+            Context lastContext = CURRENT_CONTEXT.get();
             if (lastContext == this)
                 consumer.accept(t);
             else
@@ -1506,7 +1544,7 @@ public class ContextHandler extends Handler.Wrapper implements Attributes, Alias
 
         public void run(Runnable runnable, Request request)
         {
-            Context lastContext = __context.get();
+            Context lastContext = CURRENT_CONTEXT.get();
             if (lastContext == this)
                 runnable.run();
             else
