@@ -530,6 +530,13 @@ public class HttpConnection extends AbstractMetaDataConnection implements Runnab
             if (filled <= 0)
             {
                 releaseRequestBuffer();
+                // If fillRequestBuffer() read -1, it may be because of an early EOF;
+                // force the parser check its state again so its handler can generate
+                // an error chunk if appropriate. Doing this saves returning a null
+                // chunk followed by an immediately served demand where the next read()
+                // actually makes the parser generate the error chunk.
+                if (filled < 0)
+                    _parser.parseNext(BufferUtil.EMPTY_BUFFER);
                 break;
             }
         }
@@ -753,6 +760,7 @@ public class HttpConnection extends AbstractMetaDataConnection implements Runnab
                 throw new IllegalStateException();
 
             int responseHeadersSize = getHttpConfiguration().getResponseHeaderSize();
+            int maxResponseHeadersSize = getHttpConfiguration().getMaxResponseHeaderSize();
             boolean useDirectByteBuffers = isUseOutputDirectByteBuffers();
             while (true)
             {
@@ -776,14 +784,16 @@ public class HttpConnection extends AbstractMetaDataConnection implements Runnab
                     }
                     case NEED_HEADER:
                     {
-                        _generator.setMaxHeaderBytes(getHttpConfiguration().getMaxResponseHeaderSize());
+                        int maxHeaderBytes = maxResponseHeadersSize;
+                        if (maxHeaderBytes < 0)
+                            maxHeaderBytes = responseHeadersSize;
+                        _generator.setMaxHeaderBytes(maxHeaderBytes);
                         _header = _bufferPool.acquire(responseHeadersSize, useDirectByteBuffers);
                         continue;
                     }
                     case HEADER_OVERFLOW:
                     {
-                        int maxResponseHeadersSize = getHttpConfiguration().getMaxResponseHeaderSize();
-                        if (maxResponseHeadersSize > responseHeadersSize)
+                        if (maxResponseHeadersSize > 0 && maxResponseHeadersSize > responseHeadersSize)
                         {
                             _generator.reset();
                             _header.release();
@@ -1088,8 +1098,6 @@ public class HttpConnection extends AbstractMetaDataConnection implements Runnab
                         chunk.release();
                     stream._chunk = Content.Chunk.from(bad);
                 }
-
-                ThreadPool.executeImmediately(getServer().getThreadPool(), _httpChannel.onFailure(bad));
             }
         }
     }
@@ -1214,9 +1222,7 @@ public class HttpConnection extends AbstractMetaDataConnection implements Runnab
                     {
                         HttpCompliance httpCompliance = getHttpConfiguration().getHttpCompliance();
                         if (httpCompliance.allows(MISMATCHED_AUTHORITY))
-                        {
                             getHttpChannel().getComplianceViolationListener().onComplianceViolation(new ComplianceViolation.Event(httpCompliance, MISMATCHED_AUTHORITY, _uri.asString()));
-                        }
                         else
                             throw new BadMessageException("Authority!=Host");
                     }
@@ -1244,9 +1250,7 @@ public class HttpConnection extends AbstractMetaDataConnection implements Runnab
 
             // Set path (if not already set)
             if (_uri.getPath() == null)
-            {
                 _uri.path("/");
-            }
 
             _request = new MetaData.Request(_parser.getBeginNanoTime(), _method, _uri.asImmutable(), _version, _headerBuilder, _contentLength)
             {
@@ -1270,7 +1274,6 @@ public class HttpConnection extends AbstractMetaDataConnection implements Runnab
             }
 
             boolean persistent;
-
             switch (_request.getHttpVersion())
             {
                 case HTTP_0_9:
@@ -1291,14 +1294,10 @@ public class HttpConnection extends AbstractMetaDataConnection implements Runnab
 
                     break;
                 }
-
                 case HTTP_1_1:
                 {
                     if (_unknownExpectation)
-                    {
-                        _requestHandler.badMessage(new BadMessageException(HttpStatus.EXPECTATION_FAILED_417));
-                        return null;
-                    }
+                        throw new BadMessageException(HttpStatus.EXPECTATION_FAILED_417);
 
                     persistent = getHttpConfiguration().isPersistentConnectionsEnabled() &&
                         !_connectionClose ||
@@ -1317,7 +1316,6 @@ public class HttpConnection extends AbstractMetaDataConnection implements Runnab
 
                     break;
                 }
-
                 case HTTP_2:
                 {
                     // Allow prior knowledge "upgrade" to HTTP/2 only if the connector supports h2c.
@@ -1333,7 +1331,6 @@ public class HttpConnection extends AbstractMetaDataConnection implements Runnab
                     _parser.close();
                     throw new BadMessageException(HttpStatus.UPGRADE_REQUIRED_426, "Upgrade Required");
                 }
-
                 default:
                 {
                     throw new IllegalStateException("unsupported version " + _version);
