@@ -13,9 +13,12 @@
 
 package org.eclipse.jetty.server;
 
+import java.nio.ByteBuffer;
 import java.util.Iterator;
 import java.util.List;
 import java.util.ListIterator;
+import java.util.concurrent.CompletableFuture;
+import java.util.function.Supplier;
 import java.util.stream.Stream;
 
 import org.eclipse.jetty.http.HttpCookie;
@@ -44,6 +47,8 @@ import static org.hamcrest.Matchers.is;
 import static org.hamcrest.Matchers.notNullValue;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertSame;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 
 public class ResponseTest
@@ -51,6 +56,96 @@ public class ResponseTest
     private Server server;
     private LocalConnector connector;
 
+    private static class MockResponse implements Response
+    {
+        private final Request _request;
+
+        public MockResponse(Request request)
+        {
+            _request = request;
+        }
+
+        @Override
+        public Request getRequest()
+        {
+            return _request;
+        }
+
+        @Override
+        public int getStatus()
+        {
+            return 0;
+        }
+
+        @Override
+        public void setStatus(int code)
+        {
+
+        }
+
+        @Override
+        public HttpFields.Mutable getHeaders()
+        {
+            return null;
+        }
+
+        @Override
+        public Supplier<HttpFields> getTrailersSupplier()
+        {
+            return null;
+        }
+
+        @Override
+        public void setTrailersSupplier(Supplier<HttpFields> trailers)
+        {
+
+        }
+
+        @Override
+        public boolean isCommitted()
+        {
+            return false;
+        }
+
+        @Override
+        public boolean hasLastWrite()
+        {
+            return false;
+        }
+
+        @Override
+        public boolean isCompletedSuccessfully()
+        {
+            return false;
+        }
+
+        @Override
+        public void reset()
+        {
+
+        }
+
+        @Override
+        public CompletableFuture<Void> writeInterim(int status, HttpFields headers)
+        {
+            return null;
+        }
+
+        @Override
+        public void write(boolean last, ByteBuffer byteBuffer, Callback callback)
+        {
+
+        }
+    }
+
+    private static class MockResponseWrapper extends Response.Wrapper
+    {
+        public MockResponseWrapper(Request request, Response wrapped)
+        {
+            super(request, wrapped);
+        }
+    }
+
     @BeforeEach
     public void prepare() throws Exception
     {
@@ -67,6 +162,47 @@ public class ResponseTest
         connector = null;
     }
 
+    @Test
+    public void testAsInContext() throws Exception
+    {
+        // no response
+        assertNull(Response.asInContext(null, MockResponseWrapper.class));
+
+       // response has no request
+       MockResponse mockResponse = new MockResponse(null);
+       assertNull(Response.asInContext(mockResponse, MockResponseWrapper.class));
+
+        // response with request with no context is the same as Response.as
+        MockContext mockContext = new MockContext();
+        MockRequest mockRequest = new MockRequest(mockContext);
+        mockResponse  = new MockResponse(mockRequest);
+        MockRequest.MockWrapper mockRequestWrapperA = new MockRequest.MockWrapper(mockRequest, mockContext);
+        MockResponseWrapper mockResponseWrapperA = new MockResponseWrapper(mockRequestWrapperA, mockResponse);
+        MockRequest.MockWrapper mockRequestWrapperB = new MockRequest.MockWrapper(mockRequestWrapperA, null);
+        MockResponseWrapper mockResponseWrapperB = new MockResponseWrapper(mockRequestWrapperB, mockResponseWrapperA);
+        Response asInContext = Response.asInContext(mockResponseWrapperB, MockResponseWrapper.class);
+        Response as = Response.as(mockResponseWrapperB, MockResponseWrapper.class);
+        assertSame(asInContext, as);
+        assertSame(mockResponseWrapperB, asInContext);
+        assertSame(mockResponseWrapperB, as);
+
+        // response with request with context must not cross context boundary
+        mockRequestWrapperB = new MockRequest.MockWrapper(mockRequestWrapperA, new MockContext());
+        mockResponseWrapperB = new MockResponseWrapper(mockRequestWrapperB, mockResponseWrapperA);
+        Response.Wrapper wrapperC = new Response.Wrapper(new MockRequest.MockWrapper(mockRequestWrapperB, new MockContext()), mockResponseWrapperB)
+        {
+        };
+        assertNull(Response.asInContext(wrapperC, MockResponseWrapper.class));
+
+        // response with request with same context in more than one wrap returns first matching class
+        mockRequestWrapperB = new MockRequest.MockWrapper(mockRequestWrapperA, mockContext);
+        mockResponseWrapperB = new MockResponseWrapper(mockRequestWrapperB, mockResponseWrapperA);
+        wrapperC = new Response.Wrapper(new MockRequest.Wrapper(mockRequestWrapperB), mockResponseWrapperB)
+        {
+        };
+        assertSame(mockResponseWrapperB, Response.asInContext(wrapperC, MockResponseWrapper.class));
+    }
+
     @Test
     public void testGET() throws Exception
     {
@@ -400,8 +536,8 @@ public class ResponseTest
     {
         return Stream.of(
             Arguments.of(null, "http://[bad]:xyz/", HttpStatus.FOUND_302, null),
-            Arguments.of(UriCompliance.UNSAFE, "http://[bad]:xyz/", HttpStatus.INTERNAL_SERVER_ERROR_500, "Bad authority"),
-            Arguments.of(UriCompliance.DEFAULT, "http://[bad]:xyz/", HttpStatus.INTERNAL_SERVER_ERROR_500, "Bad authority"),
+            Arguments.of(UriCompliance.UNSAFE, "http://[bad]:xyz/", HttpStatus.INTERNAL_SERVER_ERROR_500, "Bad [IPv6]"),
+            Arguments.of(UriCompliance.DEFAULT, "http://[bad]:xyz/", HttpStatus.INTERNAL_SERVER_ERROR_500, "Bad [IPv6]"),
             Arguments.of(null, "http://user:password@host.com/", HttpStatus.FOUND_302, null),
             Arguments.of(UriCompliance.DEFAULT, "http://user:password@host.com/", HttpStatus.INTERNAL_SERVER_ERROR_500, "Deprecated User Info"),
             Arguments.of(UriCompliance.LEGACY, "http://user:password@host.com/", HttpStatus.FOUND_302, null),
@@ -596,4 +732,56 @@ public class ResponseTest
             "name=test1; Domain=customized; SameSite=Lax",
             "other=test2; Domain=customized; SameSite=Lax; Attr=x"));
     }
+
+    @Test
+    public void testNullInHeaderName() throws Exception
+    {
+        server.setHandler(new Handler.Abstract()
+        {
+            @Override
+            public boolean handle(Request request, Response response, Callback callback)
+            {
+                response.setStatus(200);
+                response.getHeaders().add("X-Te\u0000st", "value");
+                Content.Sink.write(response, true, "OK", callback);
+                return true;
+            }
+        });
+        server.start();
+
+        String request = """
+                POST /test HTTP/1.0\r
+                Host: hostname\r
+                \r
+                """;
+        HttpTester.Response response = HttpTester.parseResponse(connector.getResponse(request));
+        assertEquals(HttpStatus.OK_200, response.getStatus());
+        assertThat(response.get("X-Te.st"), is("value"));
+    }
+
+    @Test
+    public void testNullInHeaderValue() throws Exception
+    {
+        server.setHandler(new Handler.Abstract()
+        {
+            @Override
+            public boolean handle(Request request, Response response, Callback callback)
+            {
+                response.setStatus(200);
+                response.getHeaders().add("X-Test", "val\u0000ue");
+                Content.Sink.write(response, true, "OK", callback);
+                return true;
+            }
+        });
+        server.start();
+
+        String request = """
+                POST /test HTTP/1.0\r
+                Host: hostname\r
+                \r
+                """;
+        HttpTester.Response response = HttpTester.parseResponse(connector.getResponse(request));
+        assertEquals(HttpStatus.OK_200, response.getStatus());
+        assertThat(response.get("X-Test"), is("val ue"));
+    }
 }
