@@ -26,6 +26,7 @@ import org.eclipse.jetty.server.Request;
 import org.eclipse.jetty.server.Response;
 import org.eclipse.jetty.server.Server;
 import org.eclipse.jetty.server.Session;
+import org.eclipse.jetty.server.handler.GracefulHandler;
 import org.eclipse.jetty.toolchain.test.MavenTestingUtils;
 import org.eclipse.jetty.util.Callback;
 import org.eclipse.jetty.util.IO;
@@ -155,7 +156,13 @@ public class SessionHandlerTest
                             out.append("Attribute ").append(name).append(" = ").append(session.getAttribute(name)).append('\n');
                         out.append("URI [")
                             .append(session.encodeURI(request, "/some/path", request.getHeaders().contains(HttpHeader.COOKIE)))
-                            .append("]");
+                            .append("]\n");
+                        out.append("RELATIVE URI [")
+                            .append(session.encodeURI(request, "../", request.getHeaders().contains(HttpHeader.COOKIE)))
+                            .append("]\n");
+                        out.append("ABSOLUTE URI [")
+                            .append(session.encodeURI(request, "http://localhost:80/foo/bar/", request.getHeaders().contains(HttpHeader.COOKIE)))
+                            .append("]\n");
                     }
                     else
                     {
@@ -488,6 +495,8 @@ public class SessionHandlerTest
             String content = response.getContent();
             assertThat(content, startsWith("Session="));
             assertThat(content, containsString("URI [/some/path;session_id=%s]".formatted(id))); // Cookies not known to be in use
+            assertThat(content, containsString("RELATIVE URI [../;session_id=%s]".formatted(id))); // Cookies not known to be in use
+            assertThat(content, containsString("ABSOLUTE URI [http://localhost:80/foo/bar/;session_id=%s]".formatted(id))); // Cookies not known to be in use
 
             // Get with cookie
             endPoint.addInput("""
@@ -503,6 +512,8 @@ public class SessionHandlerTest
             content = response.getContent();
             assertThat(content, containsString("Session=" + id.substring(0, id.indexOf(".node0"))));
             assertThat(content, containsString("URI [/some/path]")); // Cookies known to be in use
+            assertThat(content, containsString("RELATIVE URI [../]"));
+            assertThat(content, containsString("ABSOLUTE URI [http://localhost:80/foo/bar/"));
 
             // Get with parameter
             endPoint.addInput("""
@@ -517,6 +528,8 @@ public class SessionHandlerTest
             content = response.getContent();
             assertThat(content, containsString("Session=" + id.substring(0, id.indexOf(".node0"))));
             assertThat(content, containsString("URI [/some/path;session_id=%s]".formatted(id))); // Cookies not in use
+            assertThat(content, containsString("RELATIVE URI [../;session_id=%s]".formatted(id)));
+            assertThat(content, containsString("ABSOLUTE URI [http://localhost:80/foo/bar/;session_id=%s]".formatted(id)));
 
             // Get with both, but param wrong
             endPoint.addInput("""
@@ -532,6 +545,8 @@ public class SessionHandlerTest
             content = response.getContent();
             assertThat(content, containsString("Session=" + id.substring(0, id.indexOf(".node0"))));
             assertThat(content, containsString("URI [/some/path]")); // Cookies known to be in use
+            assertThat(content, containsString("RELATIVE URI [../]"));
+            assertThat(content, containsString("ABSOLUTE URI [http://localhost:80/foo/bar/]"));
 
             // Get with both, but cookie wrong
             endPoint.addInput("""
@@ -547,6 +562,8 @@ public class SessionHandlerTest
             content = response.getContent();
             assertThat(content, containsString("Session=" + id.substring(0, id.indexOf(".node0"))));
             assertThat(content, containsString("URI [/some/path]")); // Cookies known to be in use
+            assertThat(content, containsString("RELATIVE URI [../]"));
+            assertThat(content, containsString("ABSOLUTE URI [http://localhost:80/foo/bar/]"));
         }
     }
 
@@ -741,53 +758,60 @@ public class SessionHandlerTest
 
         _sessionHandler.setSessionCache(cache);
 
+        _server.insertHandler(new GracefulHandler());
+        _server.setStopTimeout(5_000);
         _server.start();
 
-        LocalConnector.LocalEndPoint endPoint = _connector.connect();
-        endPoint.addInput("""
-            GET /create HTTP/1.1
-            Host: localhost
+        String id;
+        try (LocalConnector.LocalEndPoint endPoint = _connector.connect())
+        {
+            endPoint.addInput("""
+                GET /create HTTP/1.1
+                Host: localhost
+                
+                """);
 
-            """);
+            HttpTester.Response response = HttpTester.parseResponse(endPoint.getResponse());
+            assertThat(response.getStatus(), equalTo(200));
+            String setCookie = response.get(HttpHeader.SET_COOKIE);
+            id = setCookie.substring(setCookie.indexOf("SESSION_ID=") + 11, setCookie.indexOf("; Path=/"));
+            String content = response.getContent();
+            assertThat(content, startsWith("Session="));
 
-        HttpTester.Response response = HttpTester.parseResponse(endPoint.getResponse());
-        assertThat(response.getStatus(), equalTo(200));
-        String setCookie = response.get(HttpHeader.SET_COOKIE);
-        String id = setCookie.substring(setCookie.indexOf("SESSION_ID=") + 11, setCookie.indexOf("; Path=/"));
-        String content = response.getContent();
-        assertThat(content, startsWith("Session="));
+            endPoint.addInput("""
+                GET /set/attribute/value HTTP/1.1
+                Host: localhost
+                Cookie: SESSION_ID=%s
+                
+                """.formatted(id));
 
-        endPoint.addInput("""
-            GET /set/attribute/value HTTP/1.1
-            Host: localhost
-            Cookie: SESSION_ID=%s
-
-            """.formatted(id));
-
-        response = HttpTester.parseResponse(endPoint.getResponse());
-        assertThat(response.getStatus(), equalTo(200));
-        assertThat(response.get(HttpHeader.SET_COOKIE), nullValue());
-        content = response.getContent();
-        assertThat(content, containsString("Session=" + id.substring(0, id.indexOf(".node0"))));
-        assertThat(content, containsString("attribute = value"));
+            response = HttpTester.parseResponse(endPoint.getResponse());
+            assertThat(response.getStatus(), equalTo(200));
+            assertThat(response.get(HttpHeader.SET_COOKIE), nullValue());
+            content = response.getContent();
+            assertThat(content, containsString("Session=" + id.substring(0, id.indexOf(".node0"))));
+            assertThat(content, containsString("attribute = value"));
+        }
 
         // Session should persist through restart
         _server.stop();
         _server.start();
 
-        endPoint.addInput("""
-            GET /set/attribute/value HTTP/1.1
-            Host: localhost
-            Cookie: SESSION_ID=%s
+        try (LocalConnector.LocalEndPoint endPoint = _connector.connect())
+        {
+            endPoint.addInput("""
+                GET /set/attribute/value HTTP/1.1
+                Host: localhost
+                Cookie: SESSION_ID=%s
+                
+                """.formatted(id));
 
-            """.formatted(id));
-
-        response = HttpTester.parseResponse(endPoint.getResponse());
-        assertThat(response.getStatus(), equalTo(200));
-        assertThat(response.get(HttpHeader.SET_COOKIE), nullValue());
-        content = response.getContent();
-        assertThat(content, containsString("Session=" + id.substring(0, id.indexOf(".node0"))));
-        assertThat(content, containsString("attribute = value"));
+            HttpTester.Response  response = HttpTester.parseResponse(endPoint.getResponse());
+            assertThat(response.getStatus(), equalTo(200));
+            assertThat(response.get(HttpHeader.SET_COOKIE), nullValue());
+            String content = response.getContent();
+            assertThat(content, containsString("Session=" + id.substring(0, id.indexOf(".node0"))));
+            assertThat(content, containsString("attribute = value"));
+        }
     }
-
 }
