@@ -19,6 +19,8 @@ import java.io.InterruptedIOException;
 import java.net.URI;
 import java.nio.ByteBuffer;
 import java.nio.charset.StandardCharsets;
+import java.util.ArrayList;
+import java.util.Collection;
 import java.util.List;
 import java.util.Random;
 import java.util.concurrent.CompletableFuture;
@@ -40,6 +42,8 @@ import org.eclipse.jetty.client.InputStreamResponseListener;
 import org.eclipse.jetty.client.Origin;
 import org.eclipse.jetty.client.Response;
 import org.eclipse.jetty.client.Result;
+import org.eclipse.jetty.client.StringRequestContent;
+import org.eclipse.jetty.http.HttpFields;
 import org.eclipse.jetty.http.HttpHeader;
 import org.eclipse.jetty.http.HttpMethod;
 import org.eclipse.jetty.http.HttpStatus;
@@ -63,11 +67,14 @@ import org.hamcrest.Matchers;
 import org.junit.jupiter.api.Assertions;
 import org.junit.jupiter.api.Tag;
 import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.Arguments;
 import org.junit.jupiter.params.provider.MethodSource;
 
 import static org.hamcrest.MatcherAssert.assertThat;
 import static org.hamcrest.Matchers.containsString;
+import static org.hamcrest.Matchers.equalTo;
 import static org.hamcrest.Matchers.is;
+import static org.hamcrest.Matchers.nullValue;
 import static org.junit.jupiter.api.Assertions.assertArrayEquals;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
@@ -79,6 +86,92 @@ import static org.junit.jupiter.api.Assumptions.assumeTrue;
 
 public class HttpClientTest extends AbstractTest
 {
+    @ParameterizedTest
+    @MethodSource("transports")
+    public void testClientUseContentSourceInSpawnedThreadEmptyResponseContent(Transport transport) throws Exception
+    {
+        start(transport, new Handler.Abstract()
+        {
+            @Override
+            public boolean handle(Request request, org.eclipse.jetty.server.Response response, Callback callback)
+            {
+                response.write(true, BufferUtil.EMPTY_BUFFER, callback);
+                return true;
+            }
+        });
+
+        var listener = new OnContentSourceListener();
+
+        client.newRequest(newURI(transport))
+            .method("POST")
+            .send(listener);
+
+        OnContentSourceListener.ClientResponseContent clientResponseContent = listener.clientResponseContent.get(5, TimeUnit.SECONDS);
+        assertThat(clientResponseContent.body(), is(""));
+        assertThat(clientResponseContent.status(), is(200));
+        assertThat(clientResponseContent.trailers(), nullValue());
+    }
+
+    @ParameterizedTest
+    @MethodSource("transports")
+    public void testClientUseContentSourceInSpawnedThreadWithResponseContent(Transport transport) throws Exception
+    {
+        start(transport, new Handler.Abstract()
+        {
+            @Override
+            public boolean handle(Request request, org.eclipse.jetty.server.Response response, Callback callback)
+            {
+                response.write(true, BufferUtil.toBuffer("some response content", StandardCharsets.UTF_8), callback);
+                return true;
+            }
+        });
+
+        var listener = new OnContentSourceListener();
+
+        client.newRequest(newURI(transport))
+            .method("POST")
+            .send(listener);
+
+        OnContentSourceListener.ClientResponseContent clientResponseContent = listener.clientResponseContent.get(5, TimeUnit.SECONDS);
+        assertThat(clientResponseContent.body(), is("some response content"));
+        assertThat(clientResponseContent.status(), is(200));
+        assertThat(clientResponseContent.trailers(), nullValue());
+    }
+
+    @ParameterizedTest
+    @MethodSource("transportsNoFCGI")
+    public void testClientUseContentSourceInSpawnedThreadWithTrailer(Transport transport) throws Exception
+    {
+        start(transport, new Handler.Abstract()
+        {
+            @Override
+            public boolean handle(Request request, org.eclipse.jetty.server.Response response, Callback callback) throws IOException
+            {
+                response.setTrailersSupplier(() -> HttpFields.build().add("X-Trailer-test", "foobar"));
+                // start chunked mode
+                try (Blocker.Callback blocker = Blocker.callback())
+                {
+                    response.write(false, BufferUtil.EMPTY_BUFFER, blocker);
+                    blocker.block();
+                }
+
+                response.write(true, BufferUtil.toBuffer("some response content", StandardCharsets.UTF_8), callback);
+                return true;
+            }
+        });
+
+        var listener = new OnContentSourceListener();
+
+        client.newRequest(newURI(transport))
+            .method("POST")
+            .send(listener);
+
+        OnContentSourceListener.ClientResponseContent clientResponseContent = listener.clientResponseContent.get(5, TimeUnit.SECONDS);
+        assertThat(clientResponseContent.body(), is("some response content"));
+        assertThat(clientResponseContent.status(), is(200));
+        assertThat(clientResponseContent.trailers().get("X-Trailer-test"), is("foobar"));
+    }
+
     @ParameterizedTest
     @MethodSource("transports")
     public void testRequestWithoutResponseContent(Transport transport) throws Exception
@@ -438,20 +531,22 @@ public class HttpClientTest extends AbstractTest
         });
 
         CountDownLatch latch = new CountDownLatch(1);
-        InputStreamResponseListener listener = new InputStreamResponseListener();
-        client.newRequest(newURI(transport))
-            .onResponseSuccess(response -> latch.countDown())
-            .send(listener);
-        Response response = listener.get(5, TimeUnit.SECONDS);
-        assertEquals(200, response.getStatus());
+        try (InputStreamResponseListener listener = new InputStreamResponseListener())
+        {
+            client.newRequest(newURI(transport))
+                .onResponseSuccess(response -> latch.countDown())
+                .send(listener);
+            Response response = listener.get(5, TimeUnit.SECONDS);
+            assertEquals(200, response.getStatus());
 
-        // Response cannot succeed until we read the content.
-        assertFalse(latch.await(500, TimeUnit.MILLISECONDS));
+            // Response cannot succeed until we read the content.
+            assertFalse(latch.await(500, TimeUnit.MILLISECONDS));
 
-        InputStream input = listener.getInputStream();
-        assertEquals(content, IO.toString(input));
+            InputStream input = listener.getInputStream();
+            assertEquals(content, IO.toString(input));
 
-        assertTrue(latch.await(5, TimeUnit.SECONDS));
+            assertTrue(latch.await(5, TimeUnit.SECONDS));
+        }
     }
 
     @ParameterizedTest
@@ -1044,6 +1139,183 @@ public class HttpClientTest extends AbstractTest
         assertEquals(200, response.getStatus());
     }
 
+    @ParameterizedTest
+    @MethodSource("transportsNoFCGI")
+    public void testInvalidExpectation(Transport transport) throws Exception
+    {
+        start(transport, new Handler.Abstract()
+        {
+            @Override
+            public boolean handle(Request request, org.eclipse.jetty.server.Response response, Callback callback)
+            {
+                Content.Source.consumeAll(request, callback);
+                return true;
+            }
+        });
+
+        CountDownLatch resultLatch = new CountDownLatch(1);
+        AtomicReference<Response> responseRef = new AtomicReference<>();
+        client.newRequest(newURI(transport))
+            .headers(h -> h.put(HttpHeader.EXPECT, "Invalid"))
+            // Body is necessary, otherwise the Expect header is removed.
+            .body(new StringRequestContent("hello"))
+            .onResponseHeaders(responseRef::set)
+            .send(r -> resultLatch.countDown());
+
+        // In HTTP/2, the request body is not read, as the error response
+        // is sent without calling the Handler, so a reset is triggered
+        // after the response is sent.
+        // The test verifies that the right response is received at the
+        // "headers" event, because the response body is read asynchronously
+        // and may be dropped when the RST_STREAM frame is received.
+        // Waiting for the "complete" event will likely result in a failure
+        // due to the RST_STREAM being received.
+
+        assertTrue(resultLatch.await(5, TimeUnit.SECONDS));
+        Response response = responseRef.get();
+        assertThat(response.getStatus(), equalTo(HttpStatus.EXPECTATION_FAILED_417));
+    }
+
+    public static java.util.stream.Stream<Arguments> validFieldValues()
+    {
+        List<Arguments> cases = new ArrayList<>();
+
+        Collection<Transport> transports = transports();
+        transports.remove(Transport.FCGI);
+
+        for (Transport transport : transports)
+        {
+            cases.add(Arguments.of(transport, "va ue", "va ue"));
+            cases.add(Arguments.of(transport, "va\tue", "va\tue"));
+        }
+
+        return cases.stream();
+    }
+
+    @ParameterizedTest
+    @MethodSource("validFieldValues")
+    public void testValidFieldValues(Transport transport, String rawValue, String expectedValue) throws Exception
+    {
+        start(transport, new Handler.Abstract()
+        {
+            @Override
+            public boolean handle(Request request, org.eclipse.jetty.server.Response response, Callback callback)
+            {
+                String value = request.getHeaders().get("name");
+                String msg = "name:[%s]".formatted(value);
+                Content.Sink.write(response, true, msg, callback);
+                return true;
+            }
+        });
+
+        ContentResponse response = client.newRequest(newURI(transport))
+            .method("GET")
+            .headers((headers) ->
+            {
+                headers.put("name", rawValue);
+            })
+            .send();
+
+        assertThat(response.getContentAsString(), is("name:[" + expectedValue + "]"));
+        assertThat(response.getStatus(), is(200));
+    }
+
+    public static java.util.stream.Stream<Arguments> validFieldValueContentType()
+    {
+        List<Arguments> cases = new ArrayList<>();
+
+        Collection<Transport> transports = transports();
+        transports.remove(Transport.FCGI);
+
+        for (Transport transport : transports)
+        {
+            // NOTE: this entire field value is cached.
+            cases.add(Arguments.of(transport, "text/plain; charset=UTF-8", "text/plain; charset=UTF-8"));
+            cases.add(Arguments.of(transport, "text/plain;charset=UTF-8", "text/plain;charset=UTF-8"));
+            cases.add(Arguments.of(transport, "text/plain; \tcharset=UTF-8", "text/plain; \tcharset=UTF-8"));
+        }
+
+        return cases.stream();
+    }
+
+    @ParameterizedTest
+    @MethodSource("validFieldValueContentType")
+    public void testValidFieldValueContentType(Transport transport, String rawValue, String expectedValue) throws Exception
+    {
+        start(transport, new Handler.Abstract()
+        {
+            @Override
+            public boolean handle(Request request, org.eclipse.jetty.server.Response response, Callback callback)
+            {
+                String value = request.getHeaders().get(HttpHeader.CONTENT_TYPE);
+                String msg = "content-type:[%s]".formatted(value);
+                Content.Sink.write(response, true, msg, callback);
+                return true;
+            }
+        });
+
+        ContentResponse response = client.newRequest(newURI(transport))
+            .method("GET")
+            .headers((headers) ->
+            {
+                headers.put("content-type", rawValue);
+            })
+            .send();
+
+        assertThat(response.getContentAsString(), is("content-type:[" + expectedValue + "]"));
+        assertThat(response.getStatus(), is(200));
+    }
+
+    public static java.util.stream.Stream<Arguments> invalidFieldValues()
+    {
+        List<Arguments> cases = new ArrayList<>();
+
+        Collection<Transport> transports = transports();
+        transports.remove(Transport.FCGI);
+        transports.remove(Transport.HTTP);
+        transports.remove(Transport.HTTPS);
+
+        for (Transport transport : transports)
+        {
+            cases.add(Arguments.of(transport, "\tvalue"));
+            cases.add(Arguments.of(transport, "value\t"));
+            cases.add(Arguments.of(transport, " value"));
+            cases.add(Arguments.of(transport, " value "));
+        }
+
+        return cases.stream();
+    }
+
+    @ParameterizedTest
+    @MethodSource("invalidFieldValues")
+    public void testInvalidFieldValues(Transport transport, String rawValue) throws Exception
+    {
+        start(transport, new Handler.Abstract()
+        {
+            @Override
+            public boolean handle(Request request, org.eclipse.jetty.server.Response response, Callback callback)
+            {
+                String value = request.getHeaders().get("name");
+                String msg = "name:[%s]".formatted(value);
+                Content.Sink.write(response, true, msg, callback);
+                return true;
+            }
+        });
+
+        ExecutionException exception = assertThrows(ExecutionException.class, () ->
+        {
+            client.newRequest(newURI(transport))
+                .method("GET")
+                .headers((headers) ->
+                {
+                    headers.put("name", rawValue);
+                })
+                .send();
+        });
+
+        assertThat(exception.getMessage(), containsString("Invalid 'name' header value"));
+    }
+
     private static void sleep(long time) throws IOException
     {
         try
@@ -1153,4 +1425,45 @@ public class HttpClientTest extends AbstractTest
             return latch.await(timeout, unit);
         }
     }
+
+    private static class OnContentSourceListener implements Response.Listener
+    {
+        record ClientResponseContent(int status, String body, HttpFields trailers)
+        {
+        }
+
+        final CompletableFuture<ClientResponseContent> clientResponseContent = new CompletableFuture<>();
+        final StringBuffer buffer = new StringBuffer();
+
+        @Override
+        public void onContentSource(Response response, Content.Source contentSource)
+        {
+            new Thread(() ->
+            {
+                Content.Chunk chunk = contentSource.read();
+                if (chunk == null)
+                {
+                    contentSource.demand(() -> onContentSource(response, contentSource));
+                    return;
+                }
+
+                buffer.append(BufferUtil.toString(chunk.getByteBuffer(), StandardCharsets.UTF_8));
+                chunk.release();
+
+                if (!chunk.isLast())
+                {
+                    contentSource.demand(() -> onContentSource(response, contentSource));
+                }
+                else
+                {
+                    Content.Chunk afterLastChunk = contentSource.read();
+                    if (afterLastChunk != chunk)
+                        clientResponseContent.completeExceptionally(new AssertionError("afterLastChunk != chunk"));
+                    else
+                        clientResponseContent.complete(new ClientResponseContent(response.getStatus(), buffer.toString(), response.getTrailers()));
+                }
+            }
+            ).start();
+        }
+    }
 }
