@@ -18,6 +18,7 @@ import java.io.IOException;
 import java.net.HttpURLConnection;
 import java.net.MalformedURLException;
 import java.net.URI;
+import java.net.URISyntaxException;
 import java.net.URL;
 import java.net.URLClassLoader;
 import java.nio.file.Files;
@@ -33,7 +34,7 @@ import org.eclipse.jetty.toolchain.test.jupiter.WorkDir;
 import org.eclipse.jetty.toolchain.test.jupiter.WorkDirExtension;
 import org.eclipse.jetty.util.URIUtil;
 import org.junit.jupiter.api.Assumptions;
-import org.junit.jupiter.api.Disabled;
+import org.junit.jupiter.api.Tag;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.condition.OS;
 import org.junit.jupiter.api.extension.ExtendWith;
@@ -48,7 +49,9 @@ import static org.hamcrest.Matchers.containsString;
 import static org.hamcrest.Matchers.greaterThan;
 import static org.hamcrest.Matchers.instanceOf;
 import static org.hamcrest.Matchers.is;
+import static org.hamcrest.Matchers.not;
 import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertThrows;
@@ -65,7 +68,6 @@ public class ResourceFactoryTest
         "TestData/alphabet.txt", "/TestData/alphabet.txt",
         "TestData/", "/TestData/", "TestData", "/TestData"
     })
-    @Disabled
     public void testNewClassLoaderResourceExists(String reference) throws IOException
     {
         Path alt = workDir.getEmptyPathDir().resolve("alt");
@@ -201,19 +203,27 @@ public class ResourceFactoryTest
     public void testCustomUriSchemeRegistered()
     {
         ResourceFactory.registerResourceFactory("custom", new CustomResourceFactory());
-        // Try as a normal String input
-        Resource resource = ResourceFactory.root().newResource("custom://foo");
-        assertThat(resource.getURI(), is(URI.create("custom://foo")));
-        assertThat(resource.getName(), is("custom-impl"));
+        try
+        {
+            // Try as a normal String input
+            Resource resource = ResourceFactory.root().newResource("custom://foo");
+            assertThat(resource.getURI(), is(URI.create("custom://foo")));
+            assertThat(resource.getName(), is("custom-impl"));
 
-        // Try as a formal URI object as input
-        URI uri = URI.create("custom://foo");
-        resource = ResourceFactory.root().newResource(uri);
-        assertThat(resource.getURI(), is(URI.create("custom://foo")));
-        assertThat(resource.getName(), is("custom-impl"));
+            // Try as a formal URI object as input
+            URI uri = URI.create("custom://foo");
+            resource = ResourceFactory.root().newResource(uri);
+            assertThat(resource.getURI(), is(URI.create("custom://foo")));
+            assertThat(resource.getName(), is("custom-impl"));
+        }
+        finally
+        {
+            ResourceFactory.unregisterResourceFactory("custom");
+        }
     }
 
     @Test
+    @Tag("external")
     public void testRegisterHttpsUrlFactory() throws MalformedURLException
     {
         URI uri = URI.create("https://webtide.com/");
@@ -231,20 +241,27 @@ public class ResourceFactoryTest
         }
 
         ResourceFactory.registerResourceFactory("https", new URLResourceFactory());
-        // Try as a normal String input
-        Resource resource = ResourceFactory.root().newResource("https://webtide.com/");
-        assertThat(resource.getURI(), is(URI.create("https://webtide.com/")));
-        assertThat(resource.getName(), is("https://webtide.com/"));
+        try
+        {
+            // Try as a normal String input
+            Resource resource = ResourceFactory.root().newResource("https://webtide.com/");
+            assertThat(resource.getURI(), is(URI.create("https://webtide.com/")));
+            assertThat(resource.getName(), is("https://webtide.com/"));
 
-        // Try as a formal URI object as input
-        resource = ResourceFactory.root().newResource(uri);
-        assertThat(resource.getURI(), is(URI.create("https://webtide.com/")));
-        assertThat(resource.getName(), is("https://webtide.com/"));
+            // Try as a formal URI object as input
+            resource = ResourceFactory.root().newResource(uri);
+            assertThat(resource.getURI(), is(URI.create("https://webtide.com/")));
+            assertThat(resource.getName(), is("https://webtide.com/"));
 
-        // Try a sub-resource
-        Resource subResource = resource.resolve("favicon.ico");
-        assertThat(subResource.getFileName(), is("favicon.ico"));
-        assertThat(subResource.length(), greaterThan(0L));
+            // Try a sub-resource
+            Resource subResource = resource.resolve("favicon.ico");
+            assertThat(subResource.getFileName(), is("favicon.ico"));
+            assertThat(subResource.length(), greaterThan(0L));
+        }
+        finally
+        {
+            ResourceFactory.unregisterResourceFactory("https");
+        }
     }
 
     public static Stream<Arguments> newResourceCases()
@@ -302,7 +319,7 @@ public class ResourceFactoryTest
             Path testJar = MavenPaths.findTestResourceFile("jar-file-resource.jar");
             String input = testJar.toUri().toASCIIString();
             List<Resource> resources = resourceFactory.split(input);
-            String expected = URIUtil.toJarFileUri(testJar.toUri()).toASCIIString();
+            String expected = testJar.toUri().toASCIIString();
             assertThat(resources.get(0).getURI().toString(), is(expected));
         }
     }
@@ -315,28 +332,167 @@ public class ResourceFactoryTest
             Path testJar = MavenPaths.findTestResourceFile("jar-file-resource.jar");
             String input = testJar.toString();
             List<Resource> resources = resourceFactory.split(input);
-            String expected = URIUtil.toJarFileUri(testJar.toUri()).toASCIIString();
+            String expected = testJar.toUri().toASCIIString();
             assertThat(resources.get(0).getURI().toString(), is(expected));
         }
     }
 
     @Test
-    public void testSplitOnComma()
+    public void testSplitIsAbsolute()
+    {
+        try (ResourceFactory.Closeable resourceFactory = ResourceFactory.closeable())
+        {
+            List<String> rawInputs = new ArrayList<>();
+            rawInputs.add("dir/a");
+            rawInputs.add("foo/b");
+            rawInputs.add("bar/c");
+            if (OS.WINDOWS.isCurrentOs())
+            {
+                rawInputs.add("jar:file:///C:/foo/bar.jar!/");
+                rawInputs.add("jar:file:///C:/foo/baz.jar");
+                rawInputs.add("file:///C:/foo/zed.jar");
+                rawInputs.add("C:/foo/qux.jar");
+            }
+            else
+            {
+                rawInputs.add("jar:file:///foo/bar.jar!/");
+                rawInputs.add("jar:file:///foo/baz.jar");
+                rawInputs.add("file:///foo/zed.jar");
+                rawInputs.add("/foo/qux.jar");
+            }
+
+            String rawConfig = String.join(",", rawInputs);
+            List<Resource> resources = resourceFactory.split(rawConfig, ",", true);
+            assertThat(resources.size(), is(rawInputs.size()));
+
+            for (Resource r : resources)
+            {
+                assertThat("Must not be mounted: " + r, r, not(instanceOf(MountedPathResource.class)));
+                assertThat("Must be absolute: " + r, r.getPath().isAbsolute(), is(true));
+            }
+        }
+    }
+
+    @Test
+    public void testSplitOfJavaClassPath()
+    {
+        String classPath = System.getProperty("java.class.path");
+        Assumptions.assumeTrue(classPath != null);
+
+        int classpathCount = classPath.split(File.pathSeparator).length;
+
+        try (ResourceFactory.Closeable resourceFactory = ResourceFactory.closeable())
+        {
+            List<Resource> resources = resourceFactory.split(classPath, File.pathSeparator, true);
+            assertThat(resources.size(), is(classpathCount));
+
+            for (Resource r : resources)
+            {
+                assertTrue(Resources.exists(r), "Must exist: " + r);
+                assertThat("Must not be mounted: " + r, r, not(instanceOf(MountedPathResource.class)));
+                assertThat("Must be absolute: " + r, r.getPath().isAbsolute(), is(true));
+            }
+        }
+    }
+
+    /**
+     * If a jar filesystem {@code jar:file://} URI string reference is used against a non-existent file,
+     * with unwrap turned off, then this will not result in a Resource.
+     */
+    @Test
+    public void testSplitNonExistentJarReferenceNotUnwrapNotMounted()
+    {
+        try (ResourceFactory.Closeable resourceFactory = ResourceFactory.closeable())
+        {
+            Path file = Path.of("/foo/bar.jar").toAbsolutePath();
+            assertFalse(Files.exists(file), "File should not exist: " + file);
+            String uriReference = "jar:" + file.toUri() + "!/";
+            List<Resource> resources = resourceFactory.split(uriReference, ",", false);
+            assertThat(resources.size(), is(0));
+        }
+    }
+
+    /**
+     * If a jar filesystem {@code jar:file://} URI string reference is used against a non-existent file,
+     * with unwrap turned on, then this will result in a Resource without being mounted.
+     */
+    @Test
+    public void testSplitNonExistentJarReferenceUnwrapNotMounted()
+    {
+        try (ResourceFactory.Closeable resourceFactory = ResourceFactory.closeable())
+        {
+            Path file = Path.of("/foo/bar.jar").toAbsolutePath();
+            assertFalse(Files.exists(file), "File should not exist: " + file);
+            String uriReference = "jar:" + file.toUri() + "!/";
+            List<Resource> resources = resourceFactory.split(uriReference, ",", true);
+            assertThat(resources.size(), is(1));
+            assertThat(resources.get(0), not(instanceOf(MountedPathResource.class)));
+            assertThat(resources.get(0).getPath().isAbsolute(), is(true));
+        }
+    }
+
+    /**
+     * If a normal filesystem {@code file://} URI string reference is used against a non-existent
+     * file with unwrap turned off, then this will result in a Resource reference.
+     */
+    @Test
+    public void testSplitNonExistentFileReferenceNotUnwrapNotMounted()
+    {
+        try (ResourceFactory.Closeable resourceFactory = ResourceFactory.closeable())
+        {
+            Path file = Path.of("/foo/bar.jar").toAbsolutePath();
+            assertFalse(Files.exists(file), "File should not exist: " + file);
+            String uriReference = file.toUri().toASCIIString();
+            List<Resource> resources = resourceFactory.split(uriReference, ",", false);
+            assertThat(resources.size(), is(1));
+            assertThat(resources.get(0), not(instanceOf(MountedPathResource.class)));
+            assertThat(resources.get(0).getPath().isAbsolute(), is(true));
+        }
+    }
+
+    /**
+     * If a jar filesystem {@code jar:file://} URI string reference is used against a real file that exists,
+     * with unwrap turned off, then this will result in a Resource that is mounted.
+     */
+    @Test
+    public void testSplitExistingJarFileSystemNotUnwrappedIsMounted()
+    {
+        try (ResourceFactory.Closeable resourceFactory = ResourceFactory.closeable())
+        {
+            Path jarfile = MavenPaths.findTestResourceFile("example.jar");
+            assertTrue(Files.exists(jarfile), "File should exist: " + jarfile);
+            String uriReference = "jar:" + jarfile.toUri() + "!/";
+            List<Resource> resources = resourceFactory.split(uriReference, ",", false);
+            assertThat(resources.size(), is(1));
+            assertThat(resources.get(0), instanceOf(MountedPathResource.class));
+            assertThat(resources.get(0).getPath().isAbsolute(), is(true));
+        }
+    }
+
+    @ParameterizedTest(name = "{displayName} [{index}]")
+    @ValueSource(strings = {",", "|", ";"}) // one of the standard separators
+    public void testSplitRelativePaths(String separators)
     {
         try (ResourceFactory.Closeable resourceFactory = ResourceFactory.closeable())
         {
             Path base = workDir.getEmptyPathDir();
+            System.out.println(base);
+            Assumptions.assumeFalse(base.toString().contains(separators), "Path contains separator being tested in unexpected location: " + base);
+            List<String> dirs = new ArrayList<>();
             Path dir = base.resolve("dir");
             FS.ensureDirExists(dir);
+            dirs.add(dir.toString());
             Path foo = dir.resolve("foo");
             FS.ensureDirExists(foo);
+            dirs.add(foo.toString());
             Path bar = dir.resolve("bar");
             FS.ensureDirExists(bar);
+            dirs.add(bar.toString());
 
             // This represents the user-space raw configuration
-            String config = String.format("%s,%s,%s", dir, foo, bar);
+            String config = String.join(separators, dirs);
 
-            // Split using commas
+            // Split using one of the default separators
             List<URI> uris = resourceFactory.split(config).stream().map(Resource::getURI).toList();
 
             URI[] expected = new URI[]{
@@ -349,63 +505,7 @@ public class ResourceFactoryTest
     }
 
     @Test
-    public void testSplitOnPipe()
-    {
-        try (ResourceFactory.Closeable resourceFactory = ResourceFactory.closeable())
-        {
-            Path base = workDir.getEmptyPathDir();
-            Path dir = base.resolve("dir");
-            FS.ensureDirExists(dir);
-            Path foo = dir.resolve("foo");
-            FS.ensureDirExists(foo);
-            Path bar = dir.resolve("bar");
-            FS.ensureDirExists(bar);
-
-            // This represents the user-space raw configuration
-            String config = String.format("%s|%s|%s", dir, foo, bar);
-
-            // Split using commas
-            List<URI> uris = resourceFactory.split(config).stream().map(Resource::getURI).toList();
-
-            URI[] expected = new URI[]{
-                dir.toUri(),
-                foo.toUri(),
-                bar.toUri()
-            };
-            assertThat(uris, contains(expected));
-        }
-    }
-
-    @Test
-    public void testSplitOnSemicolon()
-    {
-        try (ResourceFactory.Closeable resourceFactory = ResourceFactory.closeable())
-        {
-            Path base = workDir.getEmptyPathDir();
-            Path dir = base.resolve("dir");
-            FS.ensureDirExists(dir);
-            Path foo = dir.resolve("foo");
-            FS.ensureDirExists(foo);
-            Path bar = dir.resolve("bar");
-            FS.ensureDirExists(bar);
-
-            // This represents the user-space raw configuration
-            String config = String.format("%s;%s;%s", dir, foo, bar);
-
-            // Split using commas
-            List<URI> uris = resourceFactory.split(config).stream().map(Resource::getURI).toList();
-
-            URI[] expected = new URI[]{
-                dir.toUri(),
-                foo.toUri(),
-                bar.toUri()
-            };
-            assertThat(uris, contains(expected));
-        }
-    }
-
-    @Test
-    public void testSplitOnPathSeparatorWithGlob() throws IOException
+    public void testSplitOnPathSeparatorWithGlob() throws IOException, URISyntaxException
     {
         try (ResourceFactory.Closeable resourceFactory = ResourceFactory.closeable())
         {
@@ -427,15 +527,14 @@ public class ResourceFactoryTest
             ));
 
             // Split using commas
-            List<URI> uris = resourceFactory.split(config, File.pathSeparator).stream().map(Resource::getURI).toList();
+            List<URI> uris = resourceFactory.split(config, File.pathSeparator, false).stream().map(Resource::getURI).toList();
 
             URI[] expected = new URI[]{
                 dir.toUri(),
                 foo.toUri(),
-                // Should see the two archives as `jar:file:` URI entries
-                URIUtil.toJarFileUri(bar.resolve("lib-foo.jar").toUri()),
-                URIUtil.toJarFileUri(bar.resolve("lib-zed.zip").toUri()),
-                URIUtil.toJarFileUri(exampleJar.toUri())
+                bar.resolve("lib-foo.jar").toUri(),
+                bar.resolve("lib-zed.zip").toUri(),
+                exampleJar.toUri()
             };
 
             assertThat(uris, contains(expected));
@@ -444,7 +543,7 @@ public class ResourceFactoryTest
 
     @ParameterizedTest
     @ValueSource(strings = {";", "|", ","})
-    public void testSplitOnDelimWithGlob(String delimChar) throws IOException
+    public void testSplitOnDelimWithGlob(String delimChar) throws IOException, URISyntaxException
     {
         try (ResourceFactory.Closeable resourceFactory = ResourceFactory.closeable())
         {
@@ -473,10 +572,9 @@ public class ResourceFactoryTest
             URI[] expected = new URI[]{
                 dir.toUri(),
                 foo.toUri(),
-                // Should see the two archives as `jar:file:` URI entries
-                URIUtil.toJarFileUri(bar.resolve("lib-foo.jar").toUri()),
-                URIUtil.toJarFileUri(bar.resolve("lib-zed.zip").toUri()),
-                URIUtil.toJarFileUri(exampleJar.toUri())
+                bar.resolve("lib-foo.jar").toUri(),
+                bar.resolve("lib-zed.zip").toUri(),
+                exampleJar.toUri()
             };
 
             assertThat(uris, contains(expected));
