@@ -22,6 +22,7 @@ import java.nio.channels.WritePendingException;
 import java.time.Duration;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.concurrent.Executor;
 import java.util.concurrent.TimeoutException;
 import java.util.concurrent.atomic.AtomicInteger;
@@ -68,8 +69,8 @@ public class WebSocketCoreSession implements CoreSession, Dumpable
     private int outputBufferSize = WebSocketConstants.DEFAULT_OUTPUT_BUFFER_SIZE;
     private long maxBinaryMessageSize = WebSocketConstants.DEFAULT_MAX_BINARY_MESSAGE_SIZE;
     private long maxTextMessageSize = WebSocketConstants.DEFAULT_MAX_TEXT_MESSAGE_SIZE;
-    private Duration idleTimeout = WebSocketConstants.DEFAULT_IDLE_TIMEOUT;
-    private Duration writeTimeout = WebSocketConstants.DEFAULT_WRITE_TIMEOUT;
+    private Duration idleTimeout;
+    private Duration frameWriteTimeout = WebSocketConstants.DEFAULT_WRITE_TIMEOUT;
     private ClassLoader classLoader;
 
     public WebSocketCoreSession(FrameHandler handler, Behavior behavior, Negotiated negotiated, WebSocketComponents components)
@@ -130,13 +131,13 @@ public class WebSocketCoreSession implements CoreSession, Dumpable
     @Override
     public Duration getIdleTimeout()
     {
-        return idleTimeout;
+        return idleTimeout != null ? idleTimeout : WebSocketConstants.DEFAULT_IDLE_TIMEOUT;
     }
 
     @Override
     public void setIdleTimeout(Duration timeout)
     {
-        idleTimeout = timeout;
+        idleTimeout = Objects.requireNonNull(timeout);
         if (connection != null)
             connection.getEndPoint().setIdleTimeout(timeout.toMillis());
     }
@@ -144,15 +145,16 @@ public class WebSocketCoreSession implements CoreSession, Dumpable
     @Override
     public Duration getWriteTimeout()
     {
-        return writeTimeout;
+        return frameWriteTimeout;
     }
 
     @Override
     public void setWriteTimeout(Duration timeout)
     {
-        writeTimeout = timeout;
-        if (getConnection() != null)
-            getConnection().setWriteTimeout(timeout.toMillis());
+        frameWriteTimeout = Objects.requireNonNull(timeout);
+        WebSocketConnection connection = getConnection();
+        if (connection != null)
+            connection.setWriteTimeout(timeout.toMillis());
     }
 
     public SocketAddress getLocalAddress()
@@ -188,8 +190,14 @@ public class WebSocketCoreSession implements CoreSession, Dumpable
      */
     public void setWebSocketConnection(WebSocketConnection connection)
     {
-        connection.getEndPoint().setIdleTimeout(idleTimeout.toMillis());
-        connection.setWriteTimeout(writeTimeout.toMillis());
+        // If the idle timeout is not initialized by a WebSocket
+        // configurator, then inherit that of the EndPoint; otherwise
+        // force the EndPoint to use the WebSocket configured idle timeout.
+        if (idleTimeout == null)
+            idleTimeout = Duration.ofMillis(connection.getEndPoint().getIdleTimeout());
+        else
+            connection.getEndPoint().setIdleTimeout(idleTimeout.toMillis());
+        connection.setWriteTimeout(frameWriteTimeout.toMillis());
         extensionStack.setLastDemand(connection::demand);
         this.connection = connection;
     }
