@@ -1906,6 +1906,28 @@ public class XmlConfigurationTest
         assertThat(m.get("envB3"), is("for envB"));
     }
 
+    @Test
+    public void testMainWithDuplicateEnvironment()
+    {
+        Path base = MavenTestingUtils.getBasePath().resolve("src").resolve("test").resolve("base");
+        // Don't use environment names used in other tests, as they are stored in a static map.
+        assertThrows(IllegalArgumentException.class, () -> XmlConfiguration.main(
+            "--env",
+            "env1",
+            "-cp",
+            base.resolve("envA").toString(),
+            "--env",
+            "env2",
+            "-cp",
+            base.resolve("envB").toString(),
+            // Duplicate --env envA section is not allowed.
+            "--env",
+            "env1",
+            "-cp",
+            base.resolve("envB").toString()
+        ));
+    }
+
     @Test
     public void testPropertyNoNameAttributeWithDeprecatedAttribute() throws Exception
     {
