@@ -19,6 +19,7 @@ import java.io.OutputStream;
 import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.Enumeration;
 import java.util.List;
@@ -38,6 +39,7 @@ import jakarta.servlet.http.HttpServletRequest;
 import jakarta.servlet.http.HttpServletRequestWrapper;
 import jakarta.servlet.http.HttpServletResponse;
 import jakarta.servlet.http.HttpServletResponseWrapper;
+import jakarta.servlet.http.Part;
 import org.eclipse.jetty.ee10.servlet.util.ServletOutputStreamWrapper;
 import org.eclipse.jetty.http.HttpURI;
 import org.eclipse.jetty.http.pathmap.MatchedResource;
@@ -58,7 +60,7 @@ public class Dispatcher implements RequestDispatcher
      * Dispatch include attribute names
      */
     public static final String __FORWARD_PREFIX = "jakarta.servlet.forward.";
-    
+
     /**
      * Name of original request attribute
      */ 
@@ -66,6 +68,11 @@ public class Dispatcher implements RequestDispatcher
 
     public static final String JETTY_INCLUDE_HEADER_PREFIX = "org.eclipse.jetty.server.include.";
 
+    /**
+     * This attribute is used to store the wrapped request for internal use during a dispatch if needed.
+     */
+    public static final String WRAPPED_REQUEST_ATTRIBUTE = "org.eclipse.jetty.server.wrappedRequest";
+
     private final ServletContextHandler _contextHandler;
     private final HttpURI _uri;
     private final String _decodedPathInContext;
@@ -177,7 +184,7 @@ public class Dispatcher implements RequestDispatcher
             super(request);
         }
 
-        private Fields getParameters()
+        protected Fields getParameters()
         {
             if (_parameters == null)
             {
@@ -397,6 +404,34 @@ public class Dispatcher implements RequestDispatcher
             names.add(RequestDispatcher.FORWARD_QUERY_STRING);
             return Collections.enumeration(names);
         }
+
+        @Override
+        public Collection<Part> getParts() throws IOException, ServletException
+        {
+            try
+            {
+                setAttribute(WRAPPED_REQUEST_ATTRIBUTE, this);
+                return super.getParts();
+            }
+            finally
+            {
+                setAttribute(WRAPPED_REQUEST_ATTRIBUTE, null);
+            }
+        }
+
+        @Override
+        public Part getPart(String name) throws IOException, ServletException
+        {
+            try
+            {
+                setAttribute(WRAPPED_REQUEST_ATTRIBUTE, this);
+                return super.getPart(name);
+            }
+            finally
+            {
+                setAttribute(WRAPPED_REQUEST_ATTRIBUTE, null);
+            }
+        }
     }
 
     private class IncludeRequest extends ParameterRequestWrapper
@@ -433,6 +468,14 @@ public class Dispatcher implements RequestDispatcher
                 case RequestDispatcher.INCLUDE_REQUEST_URI -> (_uri == null) ? null : _uri.getPath();
                 case RequestDispatcher.INCLUDE_CONTEXT_PATH -> _httpServletRequest.getContextPath();
                 case RequestDispatcher.INCLUDE_QUERY_STRING -> (_uri == null) ? null : _uri.getQuery();
+                case ServletContextRequest.MULTIPART_CONFIG_ELEMENT ->
+                {
+                    // If we already have future parts, return the configuration of the wrapped request.
+                    if (super.getAttribute(ServletMultiPartFormData.class.getName()) != null)
+                        yield super.getAttribute(name);
+                    // otherwise, return the configuration of this mapping
+                    yield  _mappedServlet.getServletHolder().getMultipartConfigElement();
+                }
                 default -> super.getAttribute(name);
             };
         }
@@ -442,6 +485,11 @@ public class Dispatcher implements RequestDispatcher
         {
             //Servlet Spec 9.3.1 no include attributes if a named dispatcher
             ArrayList<String> names = new ArrayList<>(Collections.list(super.getAttributeNames()));
+
+            //only return the multipart attribute name if this servlet mapping has multipart config
+            if (names.contains(ServletContextRequest.MULTIPART_CONFIG_ELEMENT) && _mappedServlet.getServletHolder().getMultipartConfigElement() == null)
+                names.remove(ServletContextRequest.MULTIPART_CONFIG_ELEMENT);
+
             if (_named != null)
                 return Collections.enumeration(names);
             
@@ -459,6 +507,34 @@ public class Dispatcher implements RequestDispatcher
         {
             return _httpServletRequest.getQueryString();
         }
+
+        @Override
+        public Collection<Part> getParts() throws IOException, ServletException
+        {
+            try
+            {
+                setAttribute(WRAPPED_REQUEST_ATTRIBUTE, this);
+                return super.getParts();
+            }
+            finally
+            {
+                setAttribute(WRAPPED_REQUEST_ATTRIBUTE, null);
+            }
+        }
+
+        @Override
+        public Part getPart(String name) throws IOException, ServletException
+        {
+            try
+            {
+                setAttribute(WRAPPED_REQUEST_ATTRIBUTE, this);
+                return super.getPart(name);
+            }
+            finally
+            {
+                setAttribute(WRAPPED_REQUEST_ATTRIBUTE, null);
+            }
+        }
     }
 
     private static class IncludeResponse extends HttpServletResponseWrapper
@@ -723,7 +799,7 @@ public class Dispatcher implements RequestDispatcher
         @Override
         public StringBuffer getRequestURL()
         {
-            return _uri == null ? null : new StringBuffer(_uri.asString());
+            return _uri == null ? super.getRequestURL() :  new StringBuffer(HttpURI.build(_uri).query(null).scheme(super.getScheme()).host(super.getServerName()).port(super.getServerPort()).asString());
         }
 
         @Override
@@ -737,6 +813,14 @@ public class Dispatcher implements RequestDispatcher
                 case AsyncContextState.ASYNC_PATH_INFO -> _httpServletRequest.getPathInfo();
                 case AsyncContextState.ASYNC_SERVLET_PATH -> _httpServletRequest.getServletPath();
                 case AsyncContextState.ASYNC_QUERY_STRING -> _httpServletRequest.getQueryString();
+                case ServletContextRequest.MULTIPART_CONFIG_ELEMENT ->
+                {
+                    // If we already have future parts, return the configuration of the wrapped request.
+                    if (super.getAttribute(ServletMultiPartFormData.class.getName()) != null)
+                        yield super.getAttribute(name);
+                    // otherwise, return the configuration of this mapping
+                    yield  _mappedServlet.getServletHolder().getMultipartConfigElement();
+                }
                 default -> super.getAttribute(name);
             };
         }
@@ -745,6 +829,11 @@ public class Dispatcher implements RequestDispatcher
         public Enumeration<String> getAttributeNames()
         {
             ArrayList<String> names = new ArrayList<>(Collections.list(super.getAttributeNames()));
+
+            //only return the multipart attribute name if this servlet mapping has multipart config
+            if (names.contains(ServletContextRequest.MULTIPART_CONFIG_ELEMENT) && _mappedServlet.getServletHolder().getMultipartConfigElement() == null)
+                names.remove(ServletContextRequest.MULTIPART_CONFIG_ELEMENT);
+
             names.add(AsyncContextState.ASYNC_REQUEST_URI);
             names.add(AsyncContextState.ASYNC_SERVLET_PATH);
             names.add(AsyncContextState.ASYNC_PATH_INFO);
@@ -753,6 +842,65 @@ public class Dispatcher implements RequestDispatcher
             names.add(AsyncContextState.ASYNC_QUERY_STRING);
             return Collections.enumeration(names);
         }
+
+        /**
+         * Get the parameters merged from the request wrapped by this {@code AsyncRequest}
+         * and those in the {@code RequestDispatcher}.
+         * If the target path is the same as the request path, then
+         * we do not want to re-extract the query string, as the wrapped request will
+         * have already done the extraction.
+         * @return the request parameters
+         */
+        @Override
+        protected Fields getParameters()
+        {
+            // The AsyncRequest may have been dispatched to the same URI as the request that it is
+            //wrapping. If so, We should not re-extract the parameters, just use what is there.
+            if (getRequest() instanceof HttpServletRequest httpServletRequest)
+            {
+                HttpURI requestURI = HttpURI.build(httpServletRequest.getRequestURI()).query(httpServletRequest.getQueryString());
+                if (requestURI.equals(_uri))
+                {
+                    if (getRequest() instanceof ParameterRequestWrapper parameterRequestWrapper)
+                        return parameterRequestWrapper.getParameters();
+                    else if (getRequest() instanceof ServletApiRequest servletApiRequest)
+                        return servletApiRequest.getParameters();
+                    else
+                        return super.getParameters();
+                }
+                else
+                    return super.getParameters();
+            }
+            return super.getParameters();
+        }
+
+        @Override
+        public Collection<Part> getParts() throws IOException, ServletException
+        {
+            try
+            {
+                setAttribute(WRAPPED_REQUEST_ATTRIBUTE, this);
+                return super.getParts();
+            }
+            finally
+            {
+                setAttribute(WRAPPED_REQUEST_ATTRIBUTE, null);
+            }
+        }
+
+        @Override
+        public Part getPart(String name) throws IOException, ServletException
+        {
+            try
+            {
+                setAttribute(WRAPPED_REQUEST_ATTRIBUTE, this);
+                return super.getPart(name);
+            }
+            finally
+            {
+                setAttribute(WRAPPED_REQUEST_ATTRIBUTE, null);
+            }
+        }
     }
 
     private class ErrorRequest extends ParameterRequestWrapper
