@@ -20,6 +20,8 @@ import java.util.concurrent.TimeoutException;
 import java.util.concurrent.atomic.AtomicLong;
 
 import jakarta.servlet.RequestDispatcher;
+import jakarta.servlet.ServletContext;
+import jakarta.servlet.ServletException;
 import org.eclipse.jetty.ee10.servlet.ServletChannelState.Action;
 import org.eclipse.jetty.http.BadMessageException;
 import org.eclipse.jetty.http.HttpFields;
@@ -33,13 +35,13 @@ import org.eclipse.jetty.server.CustomRequestLog;
 import org.eclipse.jetty.server.HttpConfiguration;
 import org.eclipse.jetty.server.Request;
 import org.eclipse.jetty.server.Response;
-import org.eclipse.jetty.server.ResponseUtils;
 import org.eclipse.jetty.server.Server;
 import org.eclipse.jetty.server.handler.ContextHandler;
 import org.eclipse.jetty.server.handler.ContextRequest;
 import org.eclipse.jetty.util.Callback;
 import org.eclipse.jetty.util.ExceptionUtil;
 import org.eclipse.jetty.util.HostPort;
+import org.eclipse.jetty.util.StringUtil;
 import org.eclipse.jetty.util.URIUtil;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -143,7 +145,7 @@ public class ServletChannel
         if (_callback != null)
             throw new IllegalStateException();
 
-        if (request != _request && Request.as(request, ServletContextRequest.class) != _servletContextRequest)
+        if (request != _request && Request.asInContext(request, ServletContextRequest.class) != _servletContextRequest)
             throw new IllegalStateException();
         _request = request;
         _response = response;
@@ -466,14 +468,13 @@ public class ServletChannel
 
                             // The handling of the original dispatch failed, and we are now going to either generate
                             // and error response ourselves or dispatch for an error page.  If there is content left over
-                            // from the failed dispatch, then we try to consume it here and if we fail we add a
-                            // Connection:close.  This can't be deferred to COMPLETE as the response will be committed
+                            // from the failed dispatch, then we try to consume it here. If we fail, the connection may be
+                            // made non-persistent. This can't be deferred to COMPLETE as the response will be committed
                             // by then.
-                            if (!_httpInput.consumeAvailable())
-                                ResponseUtils.ensureNotPersistent(_servletContextRequest, _servletContextRequest.getServletContextResponse());
+                            _httpInput.consumeAvailable();
 
-                            ContextHandler.ScopedContext context = (ContextHandler.ScopedContext)_servletContextRequest.getAttribute(ErrorHandler.ERROR_CONTEXT);
-                            Request.Handler errorHandler = ErrorHandler.getErrorHandler(getServer(), context == null ? null : context.getContextHandler());
+                            Request.Handler errorHandler = _servletContextRequest.getAttribute(ErrorHandler.ERROR_CONTEXT) instanceof ContextHandler.ScopedContext scopedContext
+                                ? ErrorHandler.getErrorHandler(getServer(), scopedContext.getContextHandler()) : null;
 
                             // If we can't have a body or have no ErrorHandler, then create a minimal error response.
                             if (HttpStatus.hasNoBody(getServletContextResponse().getStatus()) || errorHandler == null)
@@ -556,7 +557,7 @@ public class ServletChannel
                         {
                             // Indicate Connection:close if we can't consume all.
                             if (response.getStatus() >= 200)
-                                ResponseUtils.ensureConsumeAvailableOrNotPersistent(_servletContextRequest, response);
+                                _servletContextRequest.consumeAvailable();
                         }
 
                         // RFC 7230, section 3.3.  We do this here so that a servlet error page can be sent.
@@ -699,7 +700,9 @@ public class ServletChannel
         try
         {
             _state.completing();
-            getServletContextResponse().write(true, getServletContextResponse().getHttpOutput().getByteBuffer(), Callback.from(() -> _state.completed(null), _state::completed));
+            // _state::completed may invoke async listeners, but assume non-blocking.
+            Callback callback = Callback.from(NON_BLOCKING, () -> _state.completed(null), _state::completed);
+            getServletContextResponse().write(true, getServletContextResponse().getHttpOutput().getByteBuffer(), callback);
         }
         catch (Throwable x)
         {
@@ -824,52 +827,154 @@ public class ServletChannel
         }
     }
 
+    /**
+     * An AsyncContext dispatch method was called. We may need to dispatch to
+     * a different context.
+     * @throws Exception
+     */
     public void dispatchAsync() throws Exception
     {
-        ServletContextHandler servletContextHandler = getServletContextHandler();
+        ServletContextHandler targetContextHandler = getServletContextHandler();
         ServletContextRequest servletContextRequest = getServletContextRequest();
         ServletApiRequest servletApiRequest = servletContextRequest.getServletApiRequest();
         try
         {
-            servletContextHandler.requestInitialized(servletContextRequest, servletApiRequest);
-
-            HttpURI uri;
-            String pathInContext;
             AsyncContextEvent asyncContextEvent = _state.getAsyncContextEvent();
             String dispatchString = asyncContextEvent.getDispatchPath();
-            if (dispatchString != null)
+
+            if (asyncContextEvent.getDispatchContext() instanceof CrossContextServletContext crossContextServletContext)
             {
-                String contextPath = _context.getContextPath();
-                HttpURI.Immutable dispatchUri = HttpURI.from(dispatchString);
-                pathInContext = URIUtil.canonicalPath(dispatchUri.getPath());
-                uri = HttpURI.build(servletContextRequest.getHttpURI())
-                    .path(URIUtil.addPaths(contextPath, pathInContext))
-                    .query(dispatchUri.getQuery());
+                dispatchCrossContextAsync(crossContextServletContext);
             }
-            else
+            else if (asyncContextEvent.getDispatchContext() == null)
             {
-                uri = asyncContextEvent.getBaseURI();
-                if (uri == null)
+                //the user dispatched to the current context
+                targetContextHandler.requestInitialized(servletContextRequest, servletApiRequest);
+
+                String pathInContext;
+                HttpURI uri;
+
+                if (dispatchString != null)
                 {
-                    uri = servletContextRequest.getHttpURI();
-                    pathInContext = Request.getPathInContext(servletContextRequest);
+                    //dispatch to different path in current context
+                    String contextPath = _context.getContextPath();
+                    HttpURI.Immutable dispatchUri = HttpURI.from(dispatchString);
+                    pathInContext = URIUtil.canonicalPath(dispatchUri.getPath());
+                    uri = HttpURI.build(servletContextRequest.getHttpURI())
+                        .path(URIUtil.addPaths(contextPath, pathInContext))
+                        .query(dispatchUri.getQuery());
                 }
                 else
                 {
-                    pathInContext = uri.getCanonicalPath();
-                    int length = _context.getContextPath().length();
-                    if (length > 1)
-                        pathInContext = pathInContext.substring(length);
+                    //dispatch to original path of event's request in current context
+                    uri = asyncContextEvent.getBaseURI();
+                    if (uri == null)
+                    {
+                        uri = servletContextRequest.getHttpURI();
+                        pathInContext = Request.getPathInContext(servletContextRequest);
+                    }
+                    else
+                    {
+                        pathInContext = uri.getCanonicalPath();
+                        int length = _context.getContextPath().length();
+                        if (length > 1)
+                            pathInContext = pathInContext.substring(length);
+                    }
                 }
+
+                // We first worked with the core pathInContext above, but now need to convert to servlet style
+                String decodedPathInContext = URIUtil.decodePath(pathInContext);
+                Dispatcher dispatcher = new Dispatcher(targetContextHandler, uri, decodedPathInContext);
+                dispatcher.async(asyncContextEvent.getSuppliedRequest(), asyncContextEvent.getSuppliedResponse());
+            }
+            else
+            {
+                //the container has dispatched to a different context
+                ServletContext targetContext = getServletContextHandler().getServletContext().getContext(asyncContextEvent.getDispatchContext().getContextPath());
+                if (targetContext instanceof CrossContextServletContext crossContextServletContext)
+                    dispatchCrossContextAsync(crossContextServletContext);
             }
-            // We first worked with the core pathInContext above, but now need to convert to servlet style
-            String decodedPathInContext = URIUtil.decodePath(pathInContext);
-            Dispatcher dispatcher = new Dispatcher(servletContextHandler, uri, decodedPathInContext);
-            dispatcher.async(asyncContextEvent.getSuppliedRequest(), asyncContextEvent.getSuppliedResponse());
         }
         finally
         {
-            servletContextHandler.requestDestroyed(servletContextRequest, servletApiRequest);
+            targetContextHandler.requestDestroyed(servletContextRequest, servletApiRequest);
         }
     }
+
+    /**
+     * Async has been started, but dispatched to a different context. This can happen
+     * either via application code calling {@link jakarta.servlet.AsyncContext#dispatch(ServletContext, String)}
+     * or the container dispatching after application code returns without having performed a dispatch or otherwise
+     * completed async.
+     * @param crossContextServletContext the special context containing both the origin context and the destination context
+     * @throws ServletException
+     * @throws IOException
+     */
+    private void dispatchCrossContextAsync(CrossContextServletContext crossContextServletContext) throws ServletException, IOException
+    {
+        HttpURI uri;
+        ServletContextHandler targetContextHandler = crossContextServletContext.getTargetContext().getContextHandler();
+        AsyncContextEvent asyncContextEvent = _state.getAsyncContextEvent();
+        String dispatchPathInContext = asyncContextEvent.getDispatchPath();
+
+        if (dispatchPathInContext != null)
+        {
+            //dispatch is to a specific path
+            String encodedPathQuery = URIUtil.normalizePath(URIUtil.addEncodedPaths(targetContextHandler.getContextPath(), dispatchPathInContext));
+            if (encodedPathQuery == null)
+                throw new BadMessageException(500, "Bad dispatch path");
+
+            //Add in any query params
+            if (asyncContextEvent.getBaseURI() != null)
+            {
+                HttpURI.Mutable builder = HttpURI.build(asyncContextEvent.getBaseURI(), encodedPathQuery);
+                if (StringUtil.isEmpty(builder.getParam()))
+                    builder.param(asyncContextEvent.getBaseURI().getParam());
+                if (StringUtil.isEmpty(builder.getQuery()))
+                    builder.query(asyncContextEvent.getBaseURI().getQuery());
+
+                uri = builder.asImmutable();
+                dispatchPathInContext = uri.getPathQuery();
+            }
+            else
+            {
+                //no base request URI, do our best with the context and dispatch path
+                HttpURI.Mutable mutableHttpURI = HttpURI.build(dispatchPathInContext);
+
+                String targetContextPath = targetContextHandler.getContextPath();
+                if (!StringUtil.isEmpty(targetContextPath) && !targetContextPath.equals("/"))
+                {
+                    mutableHttpURI.path(URIUtil.addPaths(targetContextPath, mutableHttpURI.getPath()));
+                }
+                uri = mutableHttpURI.asImmutable();
+            }
+        }
+        else
+        {
+            //no dispatch path, assume path as at time of start async
+            uri = asyncContextEvent.getBaseURI();
+            String contextPath;
+            if (uri == null)
+            {
+                //use the current request's uri and the current context path to extract just the servlet path
+                uri = getServletContextRequest().getHttpURI();
+                contextPath = getContext().getContextPath();
+            }
+            else
+            {
+                //use the context path at the time of start async to extract just the servlet path
+                contextPath = asyncContextEvent.getContext().getContextPath();
+            }
+            dispatchPathInContext = uri.getCanonicalPath();
+            int length = contextPath.length();
+            if (length > 1)
+                dispatchPathInContext = dispatchPathInContext.substring(length);
+            HttpURI.Mutable mutableHttpURI = HttpURI.build(dispatchPathInContext);
+            mutableHttpURI.path(URIUtil.addPaths(targetContextHandler.getContextPath(), dispatchPathInContext));
+            uri = mutableHttpURI.asImmutable();
+        }
+
+        CrossContextDispatcher dispatcher = new CrossContextDispatcher(crossContextServletContext, uri, dispatchPathInContext);
+        dispatcher.async(asyncContextEvent.getSuppliedRequest(), asyncContextEvent.getSuppliedResponse());
+    }
 }
