@@ -26,6 +26,7 @@ import java.util.concurrent.ExecutionException;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
 import java.util.function.Consumer;
+import java.util.stream.Stream;
 import java.util.zip.GZIPInputStream;
 
 import jakarta.servlet.MultipartConfigElement;
@@ -56,12 +57,16 @@ import org.eclipse.jetty.io.content.InputStreamContentSource;
 import org.eclipse.jetty.logging.StacklessLogging;
 import org.eclipse.jetty.server.Server;
 import org.eclipse.jetty.server.ServerConnector;
+import org.eclipse.jetty.server.handler.ErrorHandler;
 import org.eclipse.jetty.server.handler.gzip.GzipHandler;
+import org.eclipse.jetty.util.Blocker;
 import org.eclipse.jetty.util.IO;
 import org.eclipse.jetty.util.component.LifeCycle;
 import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.Arguments;
+import org.junit.jupiter.params.provider.MethodSource;
 import org.junit.jupiter.params.provider.ValueSource;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
@@ -95,15 +100,23 @@ public class MultiPartServletTest
 
     private void start(HttpServlet servlet, MultipartConfigElement config, boolean eager) throws Exception
     {
-        config = config == null ? new MultipartConfigElement(tmpDirString, MAX_FILE_SIZE, -1, 0) : config;
+        start(servletContextHandler ->
+        {
+            ServletHolder servletHolder = new ServletHolder(servlet);
+            servletHolder.getRegistration().setMultipartConfig(config == null
+                ? new MultipartConfigElement(tmpDirString, MAX_FILE_SIZE, -1, 0) : config);
+            servletContextHandler.addServlet(servletHolder, "/");
+        }, eager);
+    }
+
+    private void start(Consumer<ServletContextHandler> consumer, boolean eager) throws Exception
+    {
         server = new Server(null, null, null);
         connector = new ServerConnector(server);
         server.addConnector(connector);
 
         ServletContextHandler servletContextHandler = new ServletContextHandler("/");
-        ServletHolder servletHolder = new ServletHolder(servlet);
-        servletHolder.getRegistration().setMultipartConfig(config);
-        servletContextHandler.addServlet(servletHolder, "/");
+        consumer.accept(servletContextHandler);
         server.setHandler(servletContextHandler);
 
         GzipHandler gzipHandler = new GzipHandler();
@@ -147,24 +160,26 @@ public class MultiPartServletTest
         multiPart.addPart(new MultiPart.ContentSourcePart("param", null, null, content));
         multiPart.close();
 
-        InputStreamResponseListener listener = new InputStreamResponseListener();
-        client.newRequest("localhost", connector.getLocalPort())
-            .path("/defaultConfig")
-            .scheme(HttpScheme.HTTP.asString())
-            .method(HttpMethod.POST)
-            .body(multiPart)
-            .send(listener);
-
-        // Write large amount of content to the part.
-        byte[] byteArray = new byte[1024 * 1024];
-        Arrays.fill(byteArray, (byte)1);
-        for (int i = 0; i < 1024 * 2; i++)
+        try (InputStreamResponseListener listener = new InputStreamResponseListener())
         {
-            content.getOutputStream().write(byteArray);
-        }
-        content.close();
+            client.newRequest("localhost", connector.getLocalPort())
+                .path("/defaultConfig")
+                .scheme(HttpScheme.HTTP.asString())
+                .method(HttpMethod.POST)
+                .body(multiPart)
+                .send(listener);
 
-        assert400orEof(listener, responseContent -> assertThat(responseContent, containsString("400")));
+            // Write large amount of content to the part.
+            byte[] byteArray = new byte[1024 * 1024];
+            Arrays.fill(byteArray, (byte)1);
+            for (int i = 0; i < 1024 * 2; i++)
+            {
+                content.getOutputStream().write(byteArray);
+            }
+            content.close();
+
+            assert400orEof(listener, responseContent -> assertThat(responseContent, containsString("400")));
+        }
     }
 
     @ParameterizedTest
@@ -191,15 +206,17 @@ public class MultiPartServletTest
         }
         multiPart.close();
 
-        InputStreamResponseListener listener = new InputStreamResponseListener();
-        client.newRequest("localhost", connector.getLocalPort())
-            .path("/defaultConfig")
-            .scheme(HttpScheme.HTTP.asString())
-            .method(HttpMethod.POST)
-            .body(multiPart)
-            .send(listener);
+        try (InputStreamResponseListener listener = new InputStreamResponseListener())
+        {
+            client.newRequest("localhost", connector.getLocalPort())
+                .path("/defaultConfig")
+                .scheme(HttpScheme.HTTP.asString())
+                .method(HttpMethod.POST)
+                .body(multiPart)
+                .send(listener);
 
-        assert400orEof(listener, responseContent -> assertThat(responseContent, containsString("400")));
+            assert400orEof(listener, responseContent -> assertThat(responseContent, containsString("400")));
+        }
     }
 
     @ParameterizedTest
@@ -209,7 +226,7 @@ public class MultiPartServletTest
         start(new HttpServlet()
         {
             @Override
-            protected void service(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException
+            protected void service(HttpServletRequest req, HttpServletResponse resp)
             {
                 req.getParameterMap();
             }
@@ -220,56 +237,58 @@ public class MultiPartServletTest
         multiPart.addPart(new MultiPart.ContentSourcePart("param", null, null, content));
         multiPart.close();
 
-        InputStreamResponseListener listener = new InputStreamResponseListener();
-        client.newRequest("localhost", connector.getLocalPort())
-            .path("/requestSizeLimit")
-            .scheme(HttpScheme.HTTP.asString())
-            .method(HttpMethod.POST)
-            .body(multiPart)
-            .send(listener);
-
-        Throwable writeError = null;
-        try
+        try (InputStreamResponseListener listener = new InputStreamResponseListener())
         {
-            // Write large amount of content to the part.
-            byte[] byteArray = new byte[1024 * 1024];
-            Arrays.fill(byteArray, (byte)1);
-            for (int i = 0; i < 1024 * 1024; i++)
+            client.newRequest("localhost", connector.getLocalPort())
+                .path("/requestSizeLimit")
+                .scheme(HttpScheme.HTTP.asString())
+                .method(HttpMethod.POST)
+                .body(multiPart)
+                .send(listener);
+
+            Throwable writeError = null;
+            try
             {
-                content.getOutputStream().write(byteArray);
+                // Write large amount of content to the part.
+                byte[] byteArray = new byte[1024 * 1024];
+                Arrays.fill(byteArray, (byte)1);
+                for (int i = 0; i < 1024 * 1024; i++)
+                {
+                    content.getOutputStream().write(byteArray);
+                }
+                fail("We should never be able to write all the content.");
+            }
+            catch (Exception e)
+            {
+                writeError = e;
             }
-            fail("We should never be able to write all the content.");
-        }
-        catch (Exception e)
-        {
-            writeError = e;
-        }
 
-        assertThat(writeError, instanceOf(EofException.class));
+            assertThat(writeError, instanceOf(EofException.class));
 
-        assert400orEof(listener, null);
+            assert400orEof(listener, null);
+        }
     }
 
     private static void assert400orEof(InputStreamResponseListener listener, Consumer<String> checkbody) throws InterruptedException, TimeoutException
     {
         // There is a race here, either we fail trying to write some more content OR
         // we get 400 response, for some reason reading the content throws EofException.
-        String responseContent = null;
         try
         {
             Response response = listener.get(60, TimeUnit.SECONDS);
             assertThat(response.getStatus(), equalTo(HttpStatus.BAD_REQUEST_400));
-            responseContent = IO.toString(listener.getInputStream());
+            String responseContent = IO.toString(listener.getInputStream());
+            if (checkbody != null)
+                checkbody.accept(responseContent);
+        }
+        catch (EofException ignored)
+        {
         }
         catch (ExecutionException | IOException e)
         {
             Throwable cause = e.getCause();
             assertThat(cause, instanceOf(EofException.class));
-            return;
         }
-
-        if (checkbody != null)
-            checkbody.accept(responseContent);
     }
 
     @ParameterizedTest
@@ -302,7 +321,7 @@ public class MultiPartServletTest
                 --A1B2C3
                 Content-Disposition: form-data; name="part1"
                 Content-Type: text/plain; charset="UTF-8"
-                                
+                
                 content1
                 --A1B2C3--
                 """;
@@ -311,7 +330,7 @@ public class MultiPartServletTest
                 Host: localhost
                 Content-Type: multipart/form-data; boundary="A1B2C3"
                 Content-Length: $L
-                                
+                
                 """.replace("$L", String.valueOf(content.length()));
 
             output.write(header.getBytes(UTF_8));
@@ -392,7 +411,7 @@ public class MultiPartServletTest
                 --A1B2C3
                 Content-Disposition: form-data; name="part1"
                 Content-Type: text/plain; charset="UTF-8"
-                                
+                
                 content1
                 --A1B2C3--
                 """;
@@ -401,7 +420,7 @@ public class MultiPartServletTest
                 Host: localhost
                 Content-Type: multipart/form-data; boundary="A1B2C3"
                 Content-Length: $L
-                                
+                
                 """.replace("$L", String.valueOf(content.length()));
 
             output.write(header.getBytes(UTF_8));
@@ -453,30 +472,37 @@ public class MultiPartServletTest
         multiPartContent.addPart(new MultiPart.ContentSourcePart("stringPart", null, HttpFields.EMPTY, content));
         multiPartContent.close();
 
-        InputStreamResponseListener responseStream = new InputStreamResponseListener();
-        client.newRequest("localhost", connector.getLocalPort())
-            .path("/echo")
-            .scheme(HttpScheme.HTTP.asString())
-            .method(HttpMethod.POST)
-            .headers(h -> h.add(HttpHeader.ACCEPT_ENCODING, "gzip"))
-            .body(multiPartContent)
-            .send(responseStream);
+        try (InputStreamResponseListener responseStream = new InputStreamResponseListener())
+        {
+            client.newRequest("localhost", connector.getLocalPort())
+                .path("/echo")
+                .scheme(HttpScheme.HTTP.asString())
+                .method(HttpMethod.POST)
+                .headers(h -> h.add(HttpHeader.ACCEPT_ENCODING, "gzip"))
+                .body(multiPartContent)
+                .send(responseStream);
 
-        Response response = responseStream.get(5, TimeUnit.SECONDS);
-        HttpFields headers = response.getHeaders();
-        assertThat(headers.get(HttpHeader.CONTENT_TYPE), startsWith("multipart/form-data"));
-        assertThat(headers.get(HttpHeader.CONTENT_ENCODING), is("gzip"));
+            Response response = responseStream.get(5, TimeUnit.SECONDS);
+            HttpFields headers = response.getHeaders();
+            assertThat(headers.get(HttpHeader.CONTENT_TYPE), startsWith("multipart/form-data"));
+            assertThat(headers.get(HttpHeader.CONTENT_ENCODING), is("gzip"));
 
-        String contentType = headers.get(HttpHeader.CONTENT_TYPE);
-        String boundary = MultiPart.extractBoundary(contentType);
-        InputStream inputStream = new GZIPInputStream(responseStream.getInputStream());
-        MultiPartFormData.Parser formData = new MultiPartFormData.Parser(boundary);
-        formData.setMaxParts(1);
-        formData.setMaxMemoryFileSize(-1);
-        MultiPartFormData.Parts parts = formData.parse(new InputStreamContentSource(inputStream)).join();
+            String contentType = headers.get(HttpHeader.CONTENT_TYPE);
+            String boundary = MultiPart.extractBoundary(contentType);
+            InputStream inputStream = new GZIPInputStream(responseStream.getInputStream());
+            MultiPartFormData.Parser formData = new MultiPartFormData.Parser(boundary);
+            formData.setMaxParts(1);
+            formData.setMaxMemoryFileSize(-1);
 
-        assertThat(parts.size(), is(1));
-        assertThat(parts.get(0).getContentAsString(UTF_8), is(contentString));
+            try (Blocker.Promise<MultiPartFormData.Parts> promise = Blocker.promise())
+            {
+                formData.parse(new InputStreamContentSource(inputStream), promise);
+                MultiPartFormData.Parts parts = promise.block();
+
+                assertThat(parts.size(), is(1));
+                assertThat(parts.get(0).getContentAsString(UTF_8), is(contentString));
+            }
+        }
     }
 
     @ParameterizedTest
@@ -522,7 +548,7 @@ public class MultiPartServletTest
         start(new HttpServlet()
         {
             @Override
-            protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException
+            protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws IOException
             {
                 resp.setContentType("text/plain");
                 Map<String, String[]> parameterMap = req.getParameterMap();
@@ -556,4 +582,97 @@ public class MultiPartServletTest
         assertThat(responseContent, containsString("Parameter: part3=" + contentString));
         assertThat(responseContent, not(containsString("Parameter: partFileName=" + contentString)));
     }
+
+    public static Stream<Arguments> dispatchTestArgs()
+    {
+        return Stream.of(
+            Arguments.of(true, "forward"),
+            Arguments.of(false, "forward"),
+            Arguments.of(true, "include"),
+            Arguments.of(false, "include"),
+            Arguments.of(true, "async"),
+            Arguments.of(false, "async")
+        );
+    }
+
+    @ParameterizedTest
+    @MethodSource("dispatchTestArgs")
+    public void testDispatch(boolean eager, String dispatchType) throws Exception
+    {
+        start(servletContextHandler ->
+        {
+            servletContextHandler.addServlet(new HttpServlet()
+            {
+                @Override
+                protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
+                {
+                    switch (dispatchType)
+                    {
+                        case "forward" -> request.getRequestDispatcher("/multipart").forward(request, response);
+                        case "include" -> request.getRequestDispatcher("/multipart").include(request, response);
+                        case "async" ->
+                        {
+                            request.startAsync();
+                            request.getAsyncContext().dispatch("/multipart");
+                        }
+                        default -> throw new ServletException("Unknown dispatch type: " + dispatchType);
+                    }
+                }
+            }, "/");
+
+            ServletHolder servletHolder = new ServletHolder(new HttpServlet()
+            {
+                @Override
+                protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
+                {
+                    Collection<Part> parts = request.getParts();
+                    assertNotNull(parts);
+                    assertEquals(1, parts.size());
+                    Part part = parts.iterator().next();
+                    assertEquals("part1", part.getName());
+                    Collection<String> headerNames = part.getHeaderNames();
+                    assertNotNull(headerNames);
+                    assertEquals(2, headerNames.size());
+                    String content1 = IO.toString(part.getInputStream(), UTF_8);
+                    assertEquals("content1", content1);
+                    response.getWriter().print("success!");
+                }
+            });
+            servletHolder.getRegistration().setMultipartConfig(new MultipartConfigElement(tmpDirString, MAX_FILE_SIZE, -1, 0));
+            servletContextHandler.addServlet(servletHolder, "/multipart");
+        }, eager);
+
+        if (server.getErrorHandler() instanceof ErrorHandler errorHandler)
+            errorHandler.setShowStacks(true);
+
+        try (Socket socket = new Socket("localhost", connector.getLocalPort()))
+        {
+            OutputStream output = socket.getOutputStream();
+
+            String content = """
+                --A1B2C3
+                Content-Disposition: form-data; name="part1"
+                Content-Type: text/plain; charset="UTF-8"
+                
+                content1
+                --A1B2C3--
+                """;
+            String header = """
+                POST / HTTP/1.1
+                Host: localhost
+                Content-Type: multipart/form-data; boundary="A1B2C3"
+                Content-Length: $L
+                
+                """.replace("$L", String.valueOf(content.length()));
+
+            output.write(header.getBytes(UTF_8));
+            output.write(content.getBytes(UTF_8));
+            output.flush();
+
+            HttpTester.Response response = HttpTester.parseResponse(socket.getInputStream());
+            assertNotNull(response);
+            assertEquals(HttpStatus.OK_200, response.getStatus());
+            assertThat(response.getContent(), equalTo("success!"));
+        }
+    }
 }
