@@ -15,11 +15,18 @@ package org.eclipse.jetty.ee10.session;
 
 import java.io.IOException;
 import java.lang.reflect.Proxy;
+import java.util.ArrayList;
+import java.util.List;
 import java.util.concurrent.TimeUnit;
 
 import jakarta.servlet.AsyncContext;
+import jakarta.servlet.AsyncEvent;
+import jakarta.servlet.AsyncListener;
+import jakarta.servlet.ServletContext;
 import jakarta.servlet.ServletException;
 import jakarta.servlet.ServletOutputStream;
+import jakarta.servlet.ServletRequestEvent;
+import jakarta.servlet.ServletRequestListener;
 import jakarta.servlet.WriteListener;
 import jakarta.servlet.http.HttpServlet;
 import jakarta.servlet.http.HttpServletRequest;
@@ -30,7 +37,12 @@ import org.eclipse.jetty.client.ContentResponse;
 import org.eclipse.jetty.client.HttpClient;
 import org.eclipse.jetty.ee10.servlet.ServletContextHandler;
 import org.eclipse.jetty.ee10.servlet.ServletHolder;
+import org.eclipse.jetty.http.HttpTester;
 import org.eclipse.jetty.logging.StacklessLogging;
+import org.eclipse.jetty.server.HttpConfiguration;
+import org.eclipse.jetty.server.LocalConnector;
+import org.eclipse.jetty.server.Server;
+import org.eclipse.jetty.server.handler.ContextHandlerCollection;
 import org.eclipse.jetty.session.DefaultSessionCacheFactory;
 import org.eclipse.jetty.session.SessionCache;
 import org.eclipse.jetty.session.SessionDataStoreFactory;
@@ -38,9 +50,13 @@ import org.eclipse.jetty.session.test.Foo;
 import org.eclipse.jetty.session.test.FooInvocationHandler;
 import org.eclipse.jetty.session.test.TestFoo;
 import org.eclipse.jetty.session.test.TestSessionDataStoreFactory;
+import org.hamcrest.Matchers;
 import org.junit.jupiter.api.Disabled;
 import org.junit.jupiter.api.Test;
 
+import static org.hamcrest.MatcherAssert.assertThat;
+import static org.hamcrest.Matchers.containsString;
+import static org.hamcrest.Matchers.is;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
@@ -149,7 +165,131 @@ public class AsyncTest
         }
     }
 
-    @Disabled //TODO cross context not supported
+    @Test
+    public void testSimpleCrossContextAsync() throws Exception
+    {
+        //Test async cross context dispatch from context A to context B
+        Server server = new Server();
+        ContextHandlerCollection contextHandlerCollection = new ContextHandlerCollection();
+
+        final List<String> events = new ArrayList<>();
+
+        ServletContextHandler contextA = new ServletContextHandler();
+        contextA.addEventListener(new ServletRequestListener()
+        {
+            @Override
+            public void requestDestroyed(ServletRequestEvent sre)
+            {
+                events.add("Request Destroyed: " + sre.getServletRequest().getServletContext().getContextPath());
+                ServletRequestListener.super.requestDestroyed(sre);
+            }
+
+            @Override
+            public void requestInitialized(ServletRequestEvent sre)
+            {
+                events.add("Request Initialized: " + sre.getServletRequest().getServletContext().getContextPath());
+                ServletRequestListener.super.requestInitialized(sre);
+            }
+        });
+        contextA.setContextPath("/ctxA");
+        contextA.setCrossContextDispatchSupported(true);
+        final String ASYNC_FLAG_NAME = "async.flag";
+
+        ServletHolder serviceHolder = new ServletHolder("service-servlet", new HttpServlet()
+        {
+            @Override
+            protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws IOException
+            {
+                if (req.getAttribute(ASYNC_FLAG_NAME) == null)
+                {
+                    AsyncContext asyncContext = req.startAsync(req, resp);
+                    req.setAttribute(ASYNC_FLAG_NAME, new Object());
+                    asyncContext.setTimeout(100);
+                    asyncContext.addListener(new AsyncListener()
+                    {
+                        @Override
+                        public void onComplete(AsyncEvent event)
+                        {
+                            events.add("ON complete");
+                        }
+
+                        @Override
+                        public void onTimeout(AsyncEvent event)
+                        {
+                            events.add("ON timeout");
+                        }
+
+                        @Override
+                        public void onError(AsyncEvent event)
+                        {
+                            events.add("ON error");
+                        }
+
+                        @Override
+                        public void onStartAsync(AsyncEvent event)
+                        {
+                            events.add("ON startasync");
+                        }
+                    });
+                    //perform cross context dispatch
+                    ServletContext destination = req.getServletContext().getContext("/ctxB");
+                    asyncContext.dispatch(destination, "/dispatched/z");
+                }
+            }
+        });
+
+        serviceHolder.setAsyncSupported(true);
+        contextA.addServlet(serviceHolder, "/dispatcher/*");
+        contextHandlerCollection.addHandler(contextA);
+
+        ServletContextHandler contextB = new ServletContextHandler();
+        contextB.setContextPath("/ctxB");
+        contextB.setCrossContextDispatchSupported(true);
+
+        ServletHolder testHolder = new ServletHolder("test-servlet", new HttpServlet()
+        {
+            @Override
+            protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws IOException
+            {
+                resp.setCharacterEncoding("utf-8");
+                resp.setContentType("text/plain");
+                resp.getWriter().println("Dispatched to ctxB in test-servlet");
+                resp.getWriter().println(req.getQueryString());
+            }
+        });
+
+        contextB.addServlet(testHolder, "/dispatched/*");
+        contextHandlerCollection.addHandler(contextB);
+        server.setHandler(contextHandlerCollection);
+        LocalConnector connector = new LocalConnector(server);
+        connector.getConnectionFactory(HttpConfiguration.ConnectionFactory.class).getHttpConfiguration().setSendServerVersion(false);
+        connector.getConnectionFactory(HttpConfiguration.ConnectionFactory.class).getHttpConfiguration().setSendDateHeader(false);
+        server.addConnector(connector);
+
+        server.start();
+
+        try
+        {
+
+            String rawRequest = """
+                GET /ctxA/dispatcher/x?foo=bar HTTP/1.1
+                Host: local
+                Connection: close
+                            
+                """;
+
+            HttpTester.Response response = HttpTester.parseResponse(connector.getResponse(rawRequest));
+            assertThat(response.getStatus(), is(200));
+            assertThat(events, Matchers.containsInRelativeOrder("Request Initialized: /ctxA", "Request Destroyed: /ctxA"));
+            assertThat(response.getContent(), containsString("Dispatched to ctxB in test-servlet"));
+            assertThat(response.getContent(), containsString("foo=bar"));
+        }
+        finally
+        {
+            server.stop();
+        }
+    }
+
     @Test
     public void testSessionWithCrossContextAsync() throws Exception
     {
@@ -163,11 +303,13 @@ public class AsyncTest
         SessionTestSupport server = new SessionTestSupport(0, -1, -1, cacheFactory, storeFactory);
 
         ServletContextHandler contextA = server.addContext("/ctxA");
+        contextA.setCrossContextDispatchSupported(true);
         CrossContextServlet ccServlet = new CrossContextServlet();
         ServletHolder ccHolder = new ServletHolder(ccServlet);
         contextA.addServlet(ccHolder, "/*");
 
         ServletContextHandler contextB = server.addContext("/ctxB");
+        contextB.setCrossContextDispatchSupported(true);
         TestServlet testServlet = new TestServlet();
         ServletHolder testHolder = new ServletHolder(testServlet);
         contextB.addServlet(testHolder, "/*");
@@ -250,13 +392,16 @@ public class AsyncTest
         }   
     }
 
-    @Disabled //TODO cross context not supported
     @Test
     public void testSessionWithCrossContextAsyncComplete() throws Exception
     {
         // Test async dispatch from context A to context B, which then does an
-        // async write, which creates a session (in context A) and completes outside of a
-        // dispatch
+        // async write, which creates a session and completes outside of a
+        // dispatch. Note that the session will be created in context B, because
+        // requests are immutable: the request that called startAsync retains its
+        // pathInContext etc. In earlier versions of the Servlet Spec, the request
+        // will have been mutated back to it's pre-async dispatch pathInContext
+        // etc after returning to the container and concluding the async dispatch.
 
         DefaultSessionCacheFactory cacheFactory = new DefaultSessionCacheFactory();
         cacheFactory.setEvictionPolicy(SessionCache.EVICT_ON_SESSION_EXIT);
@@ -265,11 +410,13 @@ public class AsyncTest
         SessionTestSupport server = new SessionTestSupport(0, -1, -1, cacheFactory, storeFactory);
 
         ServletContextHandler contextA = server.addContext("/ctxA");
+        contextA.setCrossContextDispatchSupported(true);
         CrossContextServlet ccServlet = new CrossContextServlet();
         ServletHolder ccHolder = new ServletHolder(ccServlet);
         contextA.addServlet(ccHolder, "/*");
 
         ServletContextHandler contextB = server.addContext("/ctxB");
+        contextB.setCrossContextDispatchSupported(true);
         TestServlet testServlet = new TestServlet();
         ServletHolder testHolder = new ServletHolder(testServlet);
         contextB.addServlet(testHolder, "/*");
@@ -280,7 +427,6 @@ public class AsyncTest
 
         try (StacklessLogging stackless = new StacklessLogging(AsyncTest.class.getPackage()))
         {
-
             client.start();
             String url = "http://localhost:" + port + "/ctxA/test?action=asyncComplete";
 
@@ -292,11 +438,11 @@ public class AsyncTest
 
             assertNotNull(sessionCookie);
 
-            //session should now be evicted from the cache A after request exited
+            //session should now be evicted from the ctx B sessioncache after request exited
             String id = SessionTestSupport.extractSessionId(sessionCookie);
             Awaitility.await().atMost(30, TimeUnit.SECONDS)
-                .until(() -> !contextA.getSessionHandler().getSessionCache().contains(id));
-            assertTrue(contextA.getSessionHandler().getSessionCache().getSessionDataStore().exists(id));
+                .until(() -> !contextB.getSessionHandler().getSessionCache().contains(id));
+            assertTrue(contextB.getSessionHandler().getSessionCache().getSessionDataStore().exists(id));
         }
         finally
         {
@@ -375,6 +521,8 @@ public class AsyncTest
                     {
                         if (out.isReady())
                         {
+                            //This is a really BAD idea. You should not be creating a
+                            //new session when you are supposed to be producing output.
                             HttpSession s = request.getSession(true);
                             out.print("OK\n");
                             acontext.complete();
@@ -384,7 +532,7 @@ public class AsyncTest
                     @Override
                     public void onError(Throwable t)
                     {
-
+                        t.printStackTrace();
                     }
                 });
             }
