@@ -13,6 +13,7 @@
 
 package org.eclipse.jetty.ee10.webapp;
 
+import java.io.File;
 import java.io.IOException;
 import java.net.URI;
 import java.net.URL;
@@ -21,6 +22,7 @@ import java.nio.file.FileSystem;
 import java.nio.file.FileSystems;
 import java.nio.file.Files;
 import java.nio.file.Path;
+import java.nio.file.Paths;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -34,6 +36,7 @@ import org.eclipse.jetty.util.TypeUtil;
 import org.eclipse.jetty.util.URIUtil;
 import org.eclipse.jetty.util.component.LifeCycle;
 import org.eclipse.jetty.util.resource.FileSystemPool;
+import org.eclipse.jetty.util.resource.MountedPathResource;
 import org.eclipse.jetty.util.resource.Resource;
 import org.eclipse.jetty.util.resource.ResourceCollators;
 import org.junit.jupiter.api.AfterEach;
@@ -46,12 +49,14 @@ import static org.hamcrest.MatcherAssert.assertThat;
 import static org.hamcrest.Matchers.empty;
 import static org.hamcrest.Matchers.hasItems;
 import static org.hamcrest.Matchers.is;
+import static org.hamcrest.Matchers.notNullValue;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 @ExtendWith(WorkDirExtension.class)
 @Isolated("Access static method of FileSystemPool")
 public class MetaInfConfigurationTest
 {
-
     @BeforeEach
     public void beforeEach()
     {
@@ -530,6 +535,48 @@ public class MetaInfConfigurationTest
         return FileSystems.newFileSystem(jarUri, env);
     }
 
+    @Test
+    public void testRelativeContainerPaths() throws Exception
+    {
+        // check to make sure that the test environment is set up correctly
+        // with a relative reference
+        String classPath = System.getProperty("java.class.path");
+        String[] files = classPath.split(File.pathSeparator);
+        String relativeFile = null;
+        for (String f : files)
+        {
+            if (f.endsWith("com-acme-janb.jar"))
+                relativeFile = f;
+        }
+
+        assertThat(relativeFile, notNullValue());
+        assertFalse(Paths.get(relativeFile).isAbsolute());
+
+        // test that the relative reference is converted into a resource with absolute Path
+        MetaInfConfiguration config = new MetaInfConfiguration();
+        WebAppContext context = new WebAppContext();
+        context.setServer(new Server());
+        try
+        {
+            context.setAttribute(MetaInfConfiguration.CONTAINER_JAR_PATTERN, ".*/com-acme-janb.jar");
+            WebAppClassLoader loader = new WebAppClassLoader(context);
+            context.setClassLoader(loader);
+            config.preConfigure(context);
+
+            List<Resource> resources = context.getMetaData().getContainerResources();
+            assertThat(resources.size(), is(1));
+            assertTrue(resources.get(0).getPath().isAbsolute());
+            assertFalse(resources.get(0) instanceof MountedPathResource);
+        }
+        finally
+        {
+            config.postConfigure(context);
+            // manually stop ResourceFactory.
+            // normally this would be done via WebAppContext.stop(), but we didn't start the context.
+            LifeCycle.stop(context.getResourceFactory());
+        }
+    }
+
     /**
      * This test examines both the classpath and the module path to find container resources.
      * This test looks {@code foo-bar.janb.jar} on the classpath (it was added there by the surefire configuration
