@@ -26,12 +26,12 @@ import jakarta.servlet.ServletResponse;
 import jakarta.servlet.http.HttpServletMapping;
 import jakarta.servlet.http.HttpServletRequest;
 import jakarta.servlet.http.HttpServletResponse;
+import org.eclipse.jetty.http.BadMessageException;
 import org.eclipse.jetty.http.HttpException;
 import org.eclipse.jetty.http.HttpURI;
 import org.eclipse.jetty.http.UriCompliance;
 import org.eclipse.jetty.util.Attributes;
 import org.eclipse.jetty.util.Fields;
-import org.eclipse.jetty.util.URIUtil;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -101,8 +101,15 @@ public class Dispatcher implements RequestDispatcher
             else
             {
                 Objects.requireNonNull(_uri);
-                // Check any URI violations against the compliance for this request
-                checkUriViolations(_uri, baseRequest);
+                try
+                {
+                    // Check any URI violations against the compliance for this request
+                    checkUriViolations(_uri, baseRequest);
+                }
+                catch (IllegalStateException e)
+                {
+                    throw new BadMessageException(e.getMessage(), e);
+                }
 
                 IncludeAttributes attr = new IncludeAttributes(
                     old_attr,
@@ -166,8 +173,15 @@ public class Dispatcher implements RequestDispatcher
             else
             {
                 Objects.requireNonNull(_uri);
-                // Check any URI violations against the compliance for this request
-                checkUriViolations(_uri, baseRequest);
+                try
+                {
+                    // Check any URI violations against the compliance for this request
+                    checkUriViolations(_uri, baseRequest);
+                }
+                catch (IllegalStateException e)
+                {
+                    throw new BadMessageException(e.getMessage(), e);
+                }
 
                 // If we have already been forwarded previously, then keep using the established
                 // original value. Otherwise, this is the first forward, and we need to establish the values.
@@ -187,9 +201,8 @@ public class Dispatcher implements RequestDispatcher
                 if (query == null)
                     query = old_uri.getQuery();
 
-                String decodedPathInContext = URIUtil.decodePath(_pathInContext);
                 baseRequest.setHttpURI(HttpURI.build(old_uri, _uri.getPath(), _uri.getParam(), query));
-                baseRequest.setContext(_contextHandler.getServletContext(), decodedPathInContext);
+                baseRequest.setContext(_contextHandler.getServletContext(), _pathInContext);
 
                 if (_uri.getQuery() != null || old_uri.getQuery() != null)
                 {
@@ -212,7 +225,7 @@ public class Dispatcher implements RequestDispatcher
                     }
                 }
 
-                _contextHandler.handle(decodedPathInContext, baseRequest, (HttpServletRequest)request, (HttpServletResponse)response);
+                _contextHandler.handle(_pathInContext, baseRequest, (HttpServletRequest)request, (HttpServletResponse)response);
 
                 // If we are not async and not closed already, then close via the possibly wrapped response.
                 if (!baseRequest.getHttpChannelState().isAsync() && !baseResponse.getHttpOutput().isClosed())
