@@ -426,8 +426,10 @@ public class Request implements HttpServletRequest
                 if (StandardCharsets.UTF_8.equals(_queryEncoding) || _queryEncoding == null && UrlEncoded.ENCODING.equals(StandardCharsets.UTF_8))
                 {
                     UriCompliance uriCompliance = getHttpChannel().getHttpConfiguration().getUriCompliance();
+                    boolean allowBadPercent = uriCompliance.allows(UriCompliance.Violation.BAD_PERCENT_ENCODING);
                     boolean allowBadUtf8 = uriCompliance.allows(UriCompliance.Violation.BAD_UTF8_ENCODING);
-                    if (!UrlEncoded.decodeUtf8To(query, 0, query.length(), _queryParameters::add, allowBadUtf8))
+                    boolean allowTruncatedUtf8 = uriCompliance.allows(UriCompliance.Violation.TRUNCATED_UTF8_ENCODING);
+                    if (!UrlEncoded.decodeUtf8To(query, 0, query.length(), _queryParameters::add, allowBadPercent, allowBadUtf8, allowTruncatedUtf8))
                     {
                         ComplianceViolation.Listener complianceViolationListener = getComplianceViolationListener();
                         if (complianceViolationListener != null)
@@ -912,9 +914,12 @@ public class Request implements HttpServletRequest
     {
         if (_inputState != INPUT_NONE && _inputState != INPUT_STREAM)
             throw new IllegalStateException("READER");
+
+        // Try to write a 100 continue if it is necessary.
+        if (_inputState == INPUT_NONE && _coreRequest.getHeaders().contains(HttpHeader.EXPECT, HttpHeaderValue.CONTINUE.asString()))
+            _channel.getCoreResponse().writeInterim(HttpStatus.CONTINUE_100, HttpFields.EMPTY);
+
         _inputState = INPUT_STREAM;
-        // Try to write a 100 continue, ignoring failure result if it was not necessary.
-        _channel.getCoreResponse().writeInterim(HttpStatus.CONTINUE_100, HttpFields.EMPTY);
         return _input;
     }
 
@@ -932,22 +937,22 @@ public class Request implements HttpServletRequest
         if (fields == null)
             return Locale.getDefault();
 
-        List<String> acceptable = fields.getQualityCSV(HttpHeader.ACCEPT_LANGUAGE);
+        List<String> acceptable = fields.getQualityCSV(HttpHeader.ACCEPT_LANGUAGE)
+            .stream()
+            .filter(StringUtil::isNotBlank)
+            .toList();
 
         // handle no locale
         if (acceptable.isEmpty())
             return Locale.getDefault();
 
-        String language = acceptable.get(0);
-        language = HttpField.stripParameters(language);
-        String country = "";
-        int dash = language.indexOf('-');
-        if (dash > -1)
-        {
-            country = language.substring(dash + 1).trim();
-            language = language.substring(0, dash).trim();
-        }
-        return new Locale(language, country);
+        // return sorted list of locales, with known locales in quality order before unknown locales in quality order
+        List<Locale> locales = acceptable.stream().map(Locale::forLanguageTag).toList();
+        // Filter again, only allowing known locales
+        List<Locale> known = locales.stream().filter(MimeTypes::isKnownLocale).toList();
+        if (known.isEmpty())
+            return Locale.getDefault();
+        return known.get(0);
     }
 
     @Override
@@ -1165,8 +1170,9 @@ public class Request implements HttpServletRequest
 
         if (_reader != null && encoding.equalsIgnoreCase(_readerEncoding))
         {
-            // Try to write a 100 continue, ignoring failure result if it was not necessary.
-            _channel.getCoreResponse().writeInterim(HttpStatus.CONTINUE_100, HttpFields.EMPTY);
+            // Try to write a 100 continue if it is necessary.
+            if (_inputState == INPUT_NONE && _coreRequest.getHeaders().contains(HttpHeader.EXPECT, HttpHeaderValue.CONTINUE.asString()))
+                _channel.getCoreResponse().writeInterim(HttpStatus.CONTINUE_100, HttpFields.EMPTY);
         }
         else
         {
@@ -1939,6 +1945,8 @@ public class Request implements HttpServletRequest
         if (_async == null)
             _async = new AsyncContextState(state);
         AsyncContextEvent event = new AsyncContextEvent(_context, _async, state, this, this, getResponse());
+        //Note that we do not remember the context, httpuri or the path, as null values for these are used by
+        //ContextHandler.handleAsync to recognize a non-cross context async dispatch
         state.startAsync(event);
         return _async;
     }
@@ -1952,6 +1960,7 @@ public class Request implements HttpServletRequest
         if (_async == null)
             _async = new AsyncContextState(state);
         AsyncContextEvent event = new AsyncContextEvent(_context, _async, state, this, servletRequest, servletResponse, getHttpURI());
+        event.setDispatchPath(URIUtil.encodePath(Request.getBaseRequest(servletRequest).getPathInContext()));
         event.setDispatchContext(getServletContext());
         state.startAsync(event);
         return _async;
@@ -2189,7 +2198,18 @@ public class Request implements HttpServletRequest
         if (newQuery != null)
         {
             newQueryParams = new Fields(true);
-            UrlEncoded.decodeTo(newQuery, newQueryParams::add, UrlEncoded.ENCODING);
+            try
+            {
+                UrlEncoded.decodeTo(newQuery, newQueryParams::add, UrlEncoded.ENCODING);
+            }
+            catch (BadMessageException e)
+            {
+                throw e;
+            }
+            catch (Throwable th)
+            {
+                throw new BadMessageException("Bad query encoding", th);
+            }
         }
 
         Fields oldQueryParams = _queryParameters;
@@ -2200,6 +2220,10 @@ public class Request implements HttpServletRequest
             {
                 UrlEncoded.decodeTo(oldQuery, oldQueryParams::add, getQueryCharset());
             }
+            catch (BadMessageException e)
+            {
+                throw e;
+            }
             catch (Throwable th)
             {
                 _queryParameters = BAD_PARAMS;
