@@ -1267,7 +1267,10 @@ public class WebAppContext extends ServletContextHandler implements WebAppClassL
      */
     public void setExtraClasspath(String extraClasspath) throws IOException
     {
-        setExtraClasspath(getResourceFactory().split(extraClasspath));
+        //convert classpath into Resources. Any jar:file: references will be
+        //unwrapped, because we need the location of the file, not the contents
+        //of the file
+        setExtraClasspath(getResourceFactory().split(extraClasspath, true));
     }
 
     public void setExtraClasspath(List<Resource> extraClasspath)
@@ -1466,19 +1469,31 @@ public class WebAppContext extends ServletContextHandler implements WebAppClassL
             if (path == null)
                 return null;
 
-            // Assumption is that the resource base has been properly setup.
-            // Spec requirement is that the WAR file is interrogated first.
-            // If a WAR file is mounted, or is extracted to a temp directory,
-            // then the first entry of the resource base must be the WAR file.
-            Resource resource = WebAppContext.this.getResource(path);
-            if (Resources.missing(resource))
-                return null;
-
-            for (Resource r: resource)
+            try
             {
-                // return first entry
-                if (Resources.exists(r))
-                    return r.getURI().toURL();
+                // Assumption is that the resource base has been properly setup.
+                // Spec requirement is that the WAR file is interrogated first.
+                // If a WAR file is mounted, or is extracted to a temp directory,
+                // then the first entry of the resource base must be the WAR file.
+                Resource resource = WebAppContext.this.getResource(path);
+                if (Resources.missing(resource))
+                    return null;
+
+                for (Resource r : resource)
+                {
+                    // return first entry
+                    if (Resources.exists(r))
+                        return r.getURI().toURL();
+                }
+            }
+            catch (MalformedURLException e)
+            {
+                throw e;
+            }
+            catch (Throwable e)
+            {
+                // catch IOException, RuntimeException, and things like java.nio.fileInvalidPathException here.
+                throw (MalformedURLException)new MalformedURLException(path).initCause(e);
             }
 
             // A Resource was returned, but did not exist
